@@ -1,6 +1,6 @@
 /*
  * mpstat: per-processor statistics
- * (C) 2000-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 2000-2007 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -28,7 +28,6 @@
 #include <errno.h>
 #include <ctype.h>
 #include <sys/utsname.h>
-#include <sys/param.h>	/* for HZ */
 
 #include "mpstat.h"
 #include "common.h"
@@ -43,14 +42,16 @@
 #endif
 
 
-struct mp_stats *st_mp_cpu[DIM];
+unsigned long long uptime[3] = {0, 0, 0};
+unsigned long long uptime0[3] = {0, 0, 0};
+struct mp_stats *st_mp_cpu[3];
 /* NOTE: Use array of _char_ for bitmaps to avoid endianness problems...*/
 unsigned char *cpu_bitmap;	/* Bit 0: Global; Bit 1: 1st proc; etc. */
-struct mp_timestamp st_mp_tstamp[DIM];
-/* Nb of processors on the machine */
-int cpu_nr = 0;
+struct tm mp_tstamp[3];
 long interval = -1, count = 0;
 
+/* Nb of processors on the machine */
+int cpu_nr = 0;
 
 /*
  ***************************************************************************
@@ -88,7 +89,7 @@ void salloc_mp_cpu(int nr_cpus)
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
+   for (i = 0; i < 3; i++) {
       if ((st_mp_cpu[i] = (struct mp_stats *) malloc(MP_STATS_SIZE * nr_cpus)) == NULL) {
 	 perror("malloc");
 	 exit(4);
@@ -106,43 +107,44 @@ void salloc_mp_cpu(int nr_cpus)
 }
 
 
+/*
+ ***************************************************************************
+ * Recalculate interval based on each CPU's tick count
+ ***************************************************************************
+ */
+unsigned long long mget_per_cpu_interval(struct mp_stats *st_mp_cpu_i,
+					 struct mp_stats *st_mp_cpu_j)
+{
+   return ((st_mp_cpu_i->cpu_user + st_mp_cpu_i->cpu_nice +
+	    st_mp_cpu_i->cpu_system + st_mp_cpu_i->cpu_iowait +
+	    st_mp_cpu_i->cpu_hardirq + st_mp_cpu_i->cpu_softirq +
+	    st_mp_cpu_i->cpu_steal + st_mp_cpu_i->cpu_idle) -
+	   (st_mp_cpu_j->cpu_user + st_mp_cpu_j->cpu_nice +
+	    st_mp_cpu_j->cpu_system + st_mp_cpu_j->cpu_iowait +
+	    st_mp_cpu_j->cpu_hardirq + st_mp_cpu_j->cpu_softirq +
+	    st_mp_cpu_j->cpu_steal + st_mp_cpu_j->cpu_idle));
+}
+
+
 /*
  ***************************************************************************
  * Core function used to display statistics
  ***************************************************************************
  */
-void write_stats_core(short prev, short curr, short dis,
+void write_stats_core(int prev, int curr, int dis,
 		      char *prev_string, char *curr_string)
 {
-   struct mp_stats *st_mp_cpu_i, *st_mp_cpu_j;
-   unsigned long long itv;
+   struct mp_stats
+      *smci = st_mp_cpu[curr] + 1,
+      *smcj = st_mp_cpu[prev] + 1;
+   unsigned long long itv, pc_itv;
    int cpu;
 
-   /*
-    * Under very special circumstances, STDOUT may become unavailable,
-    * This is what we try to guess here
-    */
-   if (write(STDOUT_FILENO, "", 0) == -1) {
-      perror("stdout");
-      exit(6);
-   }
+   /* Test stdout */
+   TEST_STDOUT(STDOUT_FILENO);
 
-   /*
-    * Interval value in jiffies, multiplied by the number of proc.
-    * The interval should always be smaller than 0xffffffff (ULONG_MAX on
-    * 32-bit architectures), except perhaps if it is the interval since
-    * system startup (we want stats since boot time).
-    * Interval is and'ed with mask 0xffffffff to handle overflow conditions
-    * that may happen since uptime values are unsigned long long but are
-    * calculated as a sum of values that _may_ be unsigned long only...
-    */
-   if (!interval)
-      itv = st_mp_tstamp[curr].uptime;
-   else
-      itv = (st_mp_tstamp[curr].uptime - st_mp_tstamp[prev].uptime) & 0xffffffff;
-
-   if (!itv)	/* Paranoia checking */
-      itv = 1;
+   /* Compute time interval */
+   itv = get_interval(uptime[prev], uptime[curr]);
 
    /* Print stats */
    if (dis)
@@ -168,25 +170,16 @@ void write_stats_core(short prev, short curr, short dis,
 	     ll_sp_value(st_mp_cpu[prev]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
    }
 
-   /*
-    * Here, we reduce the interval value to one processor,
-    * using the uptime computed for proc#0.
-    */
-   if (cpu_nr > 1) {
-      if (!interval)
-	 itv = st_mp_tstamp[curr].uptime0;
-      else
-	 itv = (st_mp_tstamp[curr].uptime0 - st_mp_tstamp[prev].uptime0) & 0xffffffff;
-      if (!itv)
-	 itv = 1;
-   }
+   /* Reduce interval value to one processor */
+   if (cpu_nr > 1)
+      itv = get_interval(uptime0[prev], uptime0[curr]);
 
    if (*cpu_bitmap & 1) {
        printf(" %9.2f\n",
 	  ll_s_value(st_mp_cpu[prev]->irq, st_mp_cpu[curr]->irq, itv));
    }
 
-   for (cpu = 1; cpu <= cpu_nr; cpu++) {
+   for (cpu = 1; cpu <= cpu_nr; cpu++, smci++, smcj++) {
 
       /* Check if we want stats about this proc */
       if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))))
@@ -194,21 +187,26 @@ void write_stats_core(short prev, short curr, short dis,
 
       printf("%-11s %4d", curr_string, cpu - 1);
 
-      st_mp_cpu_i = st_mp_cpu[curr] + cpu;
-      st_mp_cpu_j = st_mp_cpu[prev] + cpu;
+      /* Recalculate itv for current proc */
+      pc_itv = mget_per_cpu_interval(smci, smcj);
 
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f %9.2f\n",
-	     ll_sp_value(st_mp_cpu_j->cpu_user,    st_mp_cpu_i->cpu_user,    itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_nice,    st_mp_cpu_i->cpu_nice,    itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_system,  st_mp_cpu_i->cpu_system,  itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_iowait,  st_mp_cpu_i->cpu_iowait,  itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_hardirq, st_mp_cpu_i->cpu_hardirq, itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_softirq, st_mp_cpu_i->cpu_softirq, itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_steal,   st_mp_cpu_i->cpu_steal,   itv),
-	     (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle) ?
-	     0.0 :
-	     ll_sp_value(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv),
-	     ll_s_value(st_mp_cpu_j->irq, st_mp_cpu_i->irq, itv));
+      if (!pc_itv)
+	 /* Current CPU is offline */
+	 printf("    0.00    0.00    0.00    0.00    0.00    0.00    0.00    0.00      0.00\n");
+      else {
+	 printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f %9.2f\n",
+		ll_sp_value(smcj->cpu_user,    smci->cpu_user,    pc_itv),
+		ll_sp_value(smcj->cpu_nice,    smci->cpu_nice,    pc_itv),
+		ll_sp_value(smcj->cpu_system,  smci->cpu_system,  pc_itv),
+		ll_sp_value(smcj->cpu_iowait,  smci->cpu_iowait,  pc_itv),
+		ll_sp_value(smcj->cpu_hardirq, smci->cpu_hardirq, pc_itv),
+		ll_sp_value(smcj->cpu_softirq, smci->cpu_softirq, pc_itv),
+		ll_sp_value(smcj->cpu_steal,   smci->cpu_steal,   pc_itv),
+		(smci->cpu_idle < smcj->cpu_idle) ?
+		0.0 :
+		ll_sp_value(smcj->cpu_idle, smci->cpu_idle, pc_itv),
+		ll_s_value(smcj->irq, smci->irq, itv));
+      }
    }
 }
 
@@ -218,7 +216,7 @@ void write_stats_core(short prev, short curr, short dis,
  * Print statistics average
  ***************************************************************************
  */
-void write_stats_avg(short curr, short dis)
+void write_stats_avg(int curr, int dis)
 {
    char string[16];
 
@@ -232,24 +230,15 @@ void write_stats_avg(short curr, short dis)
  * Print statistics
  ***************************************************************************
  */
-void write_stats(short curr, short dis, struct tm *loc_time)
+void write_stats(int curr, int dis)
 {
    char cur_time[2][16];
 
-   /*
-    * Get previous timestamp
-    * NOTE: loc_time structure must have been init'ed before!
-    */
-   loc_time->tm_hour = st_mp_tstamp[!curr].hour;
-   loc_time->tm_min  = st_mp_tstamp[!curr].minute;
-   loc_time->tm_sec  = st_mp_tstamp[!curr].second;
-   strftime(cur_time[!curr], 16, "%X", loc_time);
+   /* Get previous timestamp */
+   strftime(cur_time[!curr], 16, "%X", &(mp_tstamp[!curr]));
 
    /* Get current timestamp */
-   loc_time->tm_hour = st_mp_tstamp[curr].hour;
-   loc_time->tm_min  = st_mp_tstamp[curr].minute;
-   loc_time->tm_sec  = st_mp_tstamp[curr].second;
-   strftime(cur_time[curr], 16, "%X", loc_time);
+   strftime(cur_time[curr], 16, "%X", &(mp_tstamp[curr]));
 
    write_stats_core(!curr, curr, dis, cur_time[!curr], cur_time[curr]);
 }
@@ -260,7 +249,7 @@ void write_stats(short curr, short dis, struct tm *loc_time)
  * Read stats from /proc/stat
  ***************************************************************************
  */
-void read_proc_stat(short curr)
+void read_proc_stat(int curr)
 {
    FILE *fp;
    struct mp_stats *st_mp_cpu_i;
@@ -300,14 +289,14 @@ void read_proc_stat(short curr)
 	  * if HZ=100).
 	  * Machine uptime is multiplied by the number of processors here.
 	  */
-	 st_mp_tstamp[curr].uptime = st_mp_cpu[curr]->cpu_user +
-	                             st_mp_cpu[curr]->cpu_nice +
-	                             st_mp_cpu[curr]->cpu_system +
-	                             st_mp_cpu[curr]->cpu_idle +
-	                             st_mp_cpu[curr]->cpu_iowait +
-	    			     st_mp_cpu[curr]->cpu_hardirq +
-	    			     st_mp_cpu[curr]->cpu_softirq +
-	    			     st_mp_cpu[curr]->cpu_steal;
+	 uptime[curr] = st_mp_cpu[curr]->cpu_user +
+	                st_mp_cpu[curr]->cpu_nice +
+	                st_mp_cpu[curr]->cpu_system +
+	                st_mp_cpu[curr]->cpu_idle +
+	                st_mp_cpu[curr]->cpu_iowait +
+	    		st_mp_cpu[curr]->cpu_hardirq +
+	    		st_mp_cpu[curr]->cpu_softirq +
+	    		st_mp_cpu[curr]->cpu_steal;
       }
 
       else if (!strncmp(line, "cpu", 3)) {
@@ -333,19 +322,15 @@ void read_proc_stat(short curr)
 	    st_mp_cpu_i->cpu_softirq = cc_softirq;
 	    st_mp_cpu_i->cpu_steal   = cc_steal;
 	 }
-	 /* else:
-	  * Additional CPUs have been dynamically registered in /proc/stat.
-	  * mpstat won't crash, but the CPU stats might be false...
-	  */
+	 /* else additional CPUs have been dynamically registered in /proc/stat */
 	
-	 if (!proc_nb)
+	 if (!proc_nb && !uptime0[curr])
 	    /*
-	     * Compute uptime reduced for one proc,
-	     * using jiffies count for proc#0.
+	     * Compute uptime reduced for one proc using proc#0.
+	     * Done only if /proc/uptime was unavailable.
 	     */
-	    st_mp_tstamp[curr].uptime0 = cc_user + cc_nice + cc_system +
-	       				 cc_idle + cc_iowait + cc_hardirq +
-	    				 cc_softirq + cc_steal;
+	    uptime0[curr] = cc_user + cc_nice + cc_system + cc_idle +
+	                    cc_iowait + cc_hardirq + cc_softirq + cc_steal;
       }
 
       else if (!strncmp(line, "intr ", 5))
@@ -365,7 +350,7 @@ void read_proc_stat(short curr)
  * Read stats from /proc/interrupts
  ***************************************************************************
  */
-void read_interrupts_stat(short curr)
+void read_interrupts_stat(int curr)
 {
    FILE *fp;
    struct mp_stats *st_mp_cpu_i;
@@ -403,24 +388,29 @@ void read_interrupts_stat(short curr)
  * and display them.
  ***************************************************************************
  */
-void rw_mp_stat_loop(short dis_hdr, unsigned long lines, int rows,
-		     struct tm *loc_time)
+void rw_mpstat_loop(int dis_hdr, unsigned long lines, int rows)
 {
-   short curr = 1, dis = 1;
-
-   st_mp_tstamp[0].hour   = loc_time->tm_hour;
-   st_mp_tstamp[0].minute = loc_time->tm_min;
-   st_mp_tstamp[0].second = loc_time->tm_sec;
+   struct mp_stats *st_mp_cpu_i, *st_mp_cpu_j;
+   int cpu;
+   int curr = 1, dis = 1;
 
    /* Read stats */
+   if (cpu_nr > 1) {
+      /*
+       * Init uptime0. So if /proc/uptime cannot fill it,
+       * this will be done by /proc/stat.
+       */
+      uptime0[0] = 0;
+      readp_uptime(&(uptime0[0]));
+   }
    read_proc_stat(0);
    read_interrupts_stat(0);
 
    if (!interval) {
       /* Display since boot time */
-      st_mp_tstamp[1] = st_mp_tstamp[0];
+      mp_tstamp[1] = mp_tstamp[0];
       memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (cpu_nr + 1));
-      write_stats(0, DISP_HDR, loc_time);
+      write_stats(0, DISP_HDR);
       exit(0);
    }
 
@@ -428,23 +418,33 @@ void rw_mp_stat_loop(short dis_hdr, unsigned long lines, int rows,
    alarm_handler(0);
 
    /* Save the first stats collected. Will be used to compute the average */
-   st_mp_tstamp[2] = st_mp_tstamp[0];
+   mp_tstamp[2] = mp_tstamp[0];
+   uptime[2] = uptime[0];
+   uptime0[2] = uptime0[0];
    memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (cpu_nr + 1));
 
    pause();
 
    do {
+      /*
+       * Resetting the structure not needed since every fields will be set.
+       * Exceptions are per-CPU structures: some of them may not be filled
+       * if corresponding processor is disabled (offline).
+       */
+      for (cpu = 1; cpu <= cpu_nr; cpu++) {
+	 st_mp_cpu_i = st_mp_cpu[curr] + cpu;
+	 st_mp_cpu_j = st_mp_cpu[!curr] + cpu;
+	 *st_mp_cpu_i = *st_mp_cpu_j;
+      }
 
-      /* Resetting the structure not needed since every fields will be set */
-
-      /* Save time */
-      get_localtime(loc_time);
-
-      st_mp_tstamp[curr].hour   = loc_time->tm_hour;
-      st_mp_tstamp[curr].minute = loc_time->tm_min;
-      st_mp_tstamp[curr].second = loc_time->tm_sec;
+      /* Get time */
+      get_localtime(&(mp_tstamp[curr]));
 
       /* Read stats */
+      if (cpu_nr > 1) {
+	 uptime0[curr] = 0;
+	 readp_uptime(&(uptime0[curr]));
+      }
       read_proc_stat(curr);
       read_interrupts_stat(curr);
 
@@ -455,7 +455,7 @@ void rw_mp_stat_loop(short dis_hdr, unsigned long lines, int rows,
 	    lines %= rows;
 	 lines++;
       }
-      write_stats(curr, dis, loc_time);
+      write_stats(curr, dis);
 
       /* Flush data */
       fflush(stdout);
@@ -483,16 +483,18 @@ int main(int argc, char **argv)
 {
    int opt = 0, i;
    struct utsname header;
-   short dis_hdr = -1, opt_used = 0;
+   int dis_hdr = -1, opt_used = 0;
    unsigned long lines = 0;
    int rows = 23;
-   struct tm loc_time;
 
 #ifdef USE_NLS
    /* Init National Language Support */
    init_nls();
 #endif
 
+   /* Get HZ */
+   get_HZ();
+
    /* How many processors on this machine ? */
    cpu_nr = get_cpu_nr(~0);
 
@@ -574,14 +576,15 @@ int main(int argc, char **argv)
       interval = 0;
 
    /* Get time */
-   get_localtime(&loc_time);
+   get_localtime(&(mp_tstamp[0]));
 
    /* Get system name, release number and hostname */
    uname(&header);
-   print_gal_header(&loc_time, header.sysname, header.release, header.nodename);
+   print_gal_header(&(mp_tstamp[0]), header.sysname, header.release,
+		    header.nodename);
 
    /* Main loop */
-   rw_mp_stat_loop(dis_hdr, lines, rows, &loc_time);
+   rw_mpstat_loop(dis_hdr, lines, rows);
 
    return 0;
 }
