@@ -1,6 +1,6 @@
 /*
  * sar/sadc: report system activity
- * (C) 1999-2006 by Sebastien Godard (sysstat <at> wanadoo.fr)
+ * (C) 1999-2007 by Sebastien Godard (sysstat <at> wanadoo.fr)
  */
 
 #ifndef _SA_H
@@ -18,7 +18,7 @@
  * System activity daily file magic number
  * (will vary when file format changes)
  */
-#define SA_MAGIC	0x2169
+#define SA_MAGIC	0x216b
 
 
 /* Define activities */
@@ -118,7 +118,6 @@
 #define WANT_ALL_PIDS(m)	(((m) & S_F_X_ALL) == S_F_X_ALL)
 
 /* Output formats (O_= Output)  */
-#define S_O_NONE		0
 #define S_O_HDR_OPTION		1
 #define S_O_PPC_OPTION		2
 #define S_O_DB_OPTION		3
@@ -128,7 +127,6 @@
 /* Files */
 #define PROC		"/proc"
 #define PSTAT		"stat"
-#define MEMINFO		"/proc/meminfo"
 #define PID_STAT	"/proc/%ld/stat"
 #define SERIAL		"/proc/tty/driver/serial"
 #define FDENTRY_STATE	"/proc/sys/fs/dentry-state"
@@ -245,20 +243,20 @@ struct file_hdr {
     * No need to save DST (daylight saving time) flag, since it is not taken
     * into account by the strftime() function used to print the timestamp.
     */
-   unsigned char  sa_day			__attribute__ ((packed));
-   unsigned char  sa_month			__attribute__ ((packed));
-   unsigned char  sa_year			__attribute__ ((packed));
+   unsigned char  sa_day;
+   unsigned char  sa_month;
+   unsigned char  sa_year;
    /*
     * Size of a long integer. Useful to know the architecture on which
     * the datafile was created.
     */
-   char		  sa_sizeof_long		__attribute__ ((packed));
+   char		  sa_sizeof_long;
    /* Operating system name */
-   char           sa_sysname[UTSNAME_LEN]	__attribute__ ((packed));
+   char           sa_sysname[UTSNAME_LEN];
    /* Machine hostname */
-   char           sa_nodename[UTSNAME_LEN]	__attribute__ ((packed));
+   char           sa_nodename[UTSNAME_LEN];
    /* Operating system release number */
-   char           sa_release[UTSNAME_LEN]	__attribute__ ((packed));
+   char           sa_release[UTSNAME_LEN];
 };
 
 #define FILE_HDR_SIZE	(sizeof(struct file_hdr))
@@ -267,7 +265,7 @@ struct file_stats {
    /* --- LONG LONG --- */
    /* Machine uptime (multiplied by the # of proc) */
    unsigned long long uptime			__attribute__ ((aligned (16)));
-   /* Uptime reduced to one processor. Set *only* on SMP machines */
+   /* Uptime reduced to one processor. Always set, even on UP machines */
    unsigned long long uptime0			__attribute__ ((aligned (16)));
    unsigned long long context_swtch		__attribute__ ((aligned (16)));
    unsigned long long cpu_user			__attribute__ ((aligned (16)));
@@ -339,14 +337,14 @@ struct file_stats {
    unsigned int  nfsd_getattcnt			__attribute__ ((packed));
    /* --- CHAR --- */
    /* Record type: R_STATS or R_DUMMY */
-   unsigned char record_type			__attribute__ ((packed));
+   unsigned char record_type;
    /*
     * Time stamp: hour, minute and second.
     * Used to determine TRUE time (immutable, non locale dependent time).
     */
-   unsigned char hour		/* (0-23) */	__attribute__ ((packed));
-   unsigned char minute		/* (0-59) */	__attribute__ ((packed));
-   unsigned char second		/* (0-59) */	__attribute__ ((packed));
+   unsigned char hour;		/* (0-23) */
+   unsigned char minute;	/* (0-59) */
+   unsigned char second;	/* (0-59) */
 };
 
 #define FILE_STATS_SIZE	(sizeof(struct file_stats))
@@ -358,7 +356,6 @@ struct stats_one_cpu {
    unsigned long long per_cpu_nice		__attribute__ ((aligned (16)));
    unsigned long long per_cpu_system		__attribute__ ((aligned (16)));
    unsigned long long per_cpu_steal		__attribute__ ((aligned (16)));
-   unsigned long long pad			__attribute__ ((aligned (16)));
 };
 
 #define STATS_ONE_CPU_SIZE	(sizeof(struct stats_one_cpu))
@@ -381,26 +378,24 @@ struct pid_stats {
    unsigned long cstime				__attribute__ ((packed));
    unsigned long cnswap				__attribute__ ((packed));
    unsigned int  processor			__attribute__ ((packed));
-   unsigned char flag				__attribute__ ((packed));
-   unsigned char pad[3]				__attribute__ ((packed));
+   unsigned char flag;
+   unsigned char pad[3];
 };
 
 #define PID_STATS_SIZE	(sizeof(struct pid_stats))
 
 struct stats_serial {
-   unsigned int  rx				__attribute__ ((aligned (8)));
-   unsigned int  tx				__attribute__ ((packed));
-   unsigned int  frame				__attribute__ ((packed));
-   unsigned int  parity				__attribute__ ((packed));
-   unsigned int  brk				__attribute__ ((packed));
-   unsigned int  overrun			__attribute__ ((packed));
-   unsigned int  line				__attribute__ ((packed));
-   unsigned char pad[4]				__attribute__ ((packed));
+   unsigned int rx				__attribute__ ((aligned (4)));
+   unsigned int tx				__attribute__ ((packed));
+   unsigned int frame				__attribute__ ((packed));
+   unsigned int parity				__attribute__ ((packed));
+   unsigned int brk				__attribute__ ((packed));
+   unsigned int overrun				__attribute__ ((packed));
+   unsigned int line				__attribute__ ((packed));
 };
 
 #define STATS_SERIAL_SIZE	(sizeof(struct stats_serial))
 
-/* See linux source file linux/include/linux/netdevice.h */
 struct stats_net_dev {
    unsigned long rx_packets			__attribute__ ((aligned (8)));
    unsigned long tx_packets			__attribute__ ((aligned (8)));
@@ -437,7 +432,7 @@ struct stats_net_dev {
  * ...
  */
 struct stats_irq_cpu {
-   unsigned int interrupt			__attribute__ ((aligned (8)));
+   unsigned int interrupt			__attribute__ ((aligned (4)));
    unsigned int irq				__attribute__ ((packed));
 };
 
@@ -459,29 +454,29 @@ struct disk_stats {
 #define DISK_STATS_SIZE		(sizeof(struct disk_stats))
 
 struct stats_sum {
-   unsigned long count				__attribute__ ((aligned (8)));
-   unsigned long frmkb				__attribute__ ((packed));
-   unsigned long bufkb				__attribute__ ((packed));
-   unsigned long camkb				__attribute__ ((packed));
-   unsigned long frskb				__attribute__ ((packed));
-   unsigned long tlskb				__attribute__ ((packed));
-   unsigned long caskb				__attribute__ ((packed));
-   unsigned long dentry_stat			__attribute__ ((packed));
-   unsigned long file_used			__attribute__ ((packed));
-   unsigned long inode_used			__attribute__ ((packed));
-   unsigned long super_used			__attribute__ ((packed));
-   unsigned long dquot_used			__attribute__ ((packed));
-   unsigned long rtsig_queued			__attribute__ ((packed));
-   unsigned long sock_inuse			__attribute__ ((packed));
-   unsigned long tcp_inuse			__attribute__ ((packed));
-   unsigned long udp_inuse			__attribute__ ((packed));
-   unsigned long raw_inuse			__attribute__ ((packed));
-   unsigned long frag_inuse			__attribute__ ((packed));
-   unsigned long nr_running			__attribute__ ((packed));
-   unsigned long nr_threads			__attribute__ ((packed));
-   unsigned long load_avg_1			__attribute__ ((packed));
-   unsigned long load_avg_5			__attribute__ ((packed));
-   unsigned long load_avg_15			__attribute__ ((packed));
+   unsigned long count;
+   unsigned long frmkb;
+   unsigned long bufkb;
+   unsigned long camkb;
+   unsigned long frskb;
+   unsigned long tlskb;
+   unsigned long caskb;
+   unsigned long dentry_stat;
+   unsigned long file_used;
+   unsigned long inode_used;
+   unsigned long super_used;
+   unsigned long dquot_used;
+   unsigned long rtsig_queued;
+   unsigned long sock_inuse;
+   unsigned long tcp_inuse;
+   unsigned long udp_inuse;
+   unsigned long raw_inuse;
+   unsigned long frag_inuse;
+   unsigned long nr_running;
+   unsigned long nr_threads;
+   unsigned long load_avg_1;
+   unsigned long load_avg_5;
+   unsigned long load_avg_15;
 };
 
 #define STATS_SUM_SIZE	(sizeof(struct stats_sum))
@@ -510,52 +505,58 @@ struct tstamp {
 #define SREALLOC(S, TYPE, SIZE)	do {							\
    				   TYPE *p;						\
 				   p = S;						\
-   				   if ((S = (TYPE *) realloc(S, (SIZE))) == NULL) {	\
-				      perror("realloc");				\
-				      exit(4);						\
+   				   if (SIZE) {						\
+   				      if ((S = (TYPE *) realloc(S, (SIZE))) == NULL) {	\
+				         perror("realloc");				\
+				         exit(4);					\
+				      }							\
+				      /* If the ptr was null, then it's a malloc() */	\
+   				      if (!p)						\
+      				         memset(S, 0, (SIZE));				\
 				   }							\
-   				   if (!p)						\
-      				      memset(S, 0, (SIZE));				\
 				} while (0)
 
 /* Functions */
-extern int	    check_disk_reg(struct file_hdr *, struct disk_stats * [],
-				   short, short, int);
+extern int   check_disk_reg(struct file_hdr *, struct disk_stats * [],
+			    int, int, int);
 extern unsigned int check_iface_reg(struct file_hdr *,
 				    struct stats_net_dev * [],
-				    short, short, unsigned int);
-extern int	    datecmp(struct tm *, struct tstamp *);
+				    int, int, unsigned int);
+extern int   datecmp(struct tm *, struct tstamp *);
 unsigned long long  get_per_cpu_interval(struct stats_one_cpu *,
 					 struct stats_one_cpu *);
-extern char	   *get_devname(unsigned int, unsigned int, int);
-extern void	    init_bitmap(unsigned char [], unsigned char, unsigned int);
-extern void	    init_stats(struct file_stats [], unsigned int [][NR_IRQS]);
-extern int	    next_slice(unsigned long long, unsigned long long,
-			       struct file_hdr *, int, long);
-extern int	    parse_sar_opt(char * [], int, unsigned int *,
-				  unsigned int *, short *, int,
-				  unsigned char [], unsigned char []);
-extern int	    parse_sar_I_opt(char * [], int *, unsigned int *, short *,
-				    unsigned char []);
-extern int	    parse_sa_P_opt(char * [], int *, unsigned int *, short *,
-				   unsigned char []);
-extern int	    parse_sar_n_opt(char * [], int *, unsigned int *, short *);
-extern int	    parse_timestamp(char * [], int *, struct tstamp *,
-				    const char *);
-extern void	    prep_file_for_reading(int *, char *, struct file_hdr *,
-					  unsigned int *, unsigned int);
-extern void	    get_itv_value(struct file_stats *, struct file_stats *,
-				  unsigned int, unsigned long long *,
-				  unsigned long long *);
-extern void	    print_report_hdr(unsigned short, unsigned int,
-				     struct tm *, struct file_hdr *);
-extern int	    sa_fread(int, void *, int, int);
-extern void	    salloc_cpu_array(struct stats_one_cpu * [], unsigned int);
-extern void	    salloc_disk_array(struct disk_stats * [], int);
-extern void	    salloc_irqcpu_array(struct stats_irq_cpu * [],
-					unsigned int, unsigned int);
-extern void	    salloc_net_dev_array(struct stats_net_dev * [],
-					 unsigned int);
-extern void	    salloc_serial_array(struct stats_serial * [], int);
+extern char *get_devname(unsigned int, unsigned int, int);
+extern void  init_bitmap(unsigned char [], unsigned char, unsigned int);
+extern void  init_stats(struct file_stats [], unsigned int [][NR_IRQS]);
+extern int   next_slice(unsigned long long, unsigned long long,
+			int, long);
+extern int   parse_sar_opt(char * [], int, unsigned int *,
+			   unsigned int *, int,
+			   unsigned char [], unsigned char []);
+extern int   parse_sar_I_opt(char * [], int *, unsigned int *,
+			     unsigned char []);
+extern int   parse_sa_P_opt(char * [], int *, unsigned int *,
+			    unsigned char []);
+extern int   parse_sar_n_opt(char * [], int *, unsigned int *);
+extern int   parse_timestamp(char * [], int *, struct tstamp *,
+			     const char *);
+extern void  prep_file_for_reading(int *, char *, struct file_hdr *,
+				   unsigned int *, unsigned int);
+extern void  get_itv_value(struct file_stats *, struct file_stats *,
+			   unsigned int, unsigned long long *,
+			   unsigned long long *);
+extern void  print_report_hdr(unsigned int, struct tm *,
+			      struct file_hdr *);
+extern int   sa_fread(int, void *, int, int);
+extern void  salloc_cpu_array(struct stats_one_cpu * [], unsigned int);
+extern void  salloc_disk_array(struct disk_stats * [], int);
+extern void  salloc_irqcpu_array(struct stats_irq_cpu * [],
+				 unsigned int, unsigned int);
+extern void  salloc_net_dev_array(struct stats_net_dev * [],
+				  unsigned int);
+extern void  salloc_serial_array(struct stats_serial * [], int);
+extern void  set_default_file(struct tm *, char *);
+extern void  set_hdr_rectime(unsigned int, struct tm *,
+			     struct file_hdr *);
 
 #endif  /* _SA_H */
