@@ -1,6 +1,6 @@
 /*
  * sar and sadf common routines.
- * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2007 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -67,7 +67,7 @@ void init_stats(struct file_stats file_stats[],
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
+   for (i = 0; i < 3; i++) {
       memset(&file_stats[i], 0, FILE_STATS_SIZE);
       memset(interrupts[i], 0, STATS_ONE_IRQ_SIZE);
    }
@@ -84,7 +84,7 @@ void salloc_cpu_array(struct stats_one_cpu *st_cpu[], unsigned int nr_cpu)
 {
    int i;
 
-   for (i = 0; i < DIM; i++)
+   for (i = 0; i < 3; i++)
       SREALLOC(st_cpu[i], struct stats_one_cpu, STATS_ONE_CPU_SIZE * nr_cpu);
 }
 
@@ -98,7 +98,7 @@ void salloc_serial_array(struct stats_serial *st_serial[], int nr_serial)
 {
    int i;
 
-   for (i = 0; i < DIM; i++)
+   for (i = 0; i < 3; i++)
       SREALLOC(st_serial[i], struct stats_serial, STATS_SERIAL_SIZE * nr_serial);
 }
 
@@ -113,7 +113,7 @@ void salloc_irqcpu_array(struct stats_irq_cpu *st_irq_cpu[],
 {
    int i;
 
-   for (i = 0; i < DIM; i++)
+   for (i = 0; i < 3; i++)
       SREALLOC(st_irq_cpu[i], struct stats_irq_cpu,
 	       STATS_IRQ_CPU_SIZE * nr_cpus * nr_irqcpu);
 }
@@ -129,7 +129,7 @@ void salloc_net_dev_array(struct stats_net_dev *st_net_dev[],
 {
    int i;
 
-   for (i = 0; i < DIM; i++)
+   for (i = 0; i < 3; i++)
       SREALLOC(st_net_dev[i], struct stats_net_dev, STATS_NET_DEV_SIZE * nr_iface);
 }
 
@@ -143,7 +143,7 @@ void salloc_disk_array(struct disk_stats *st_disk[], int nr_disk)
 {
    int i;
 
-   for (i = 0; i < DIM; i++)
+   for (i = 0; i < 3; i++)
       SREALLOC(st_disk[i], struct disk_stats, DISK_STATS_SIZE * nr_disk);
 }
 
@@ -164,7 +164,9 @@ char *get_devname(unsigned int major, unsigned int minor, int pretty)
 
    if (!pretty)
       return (buf);
-   if ((name = ioc_name(major, minor)) == NULL)
+
+   name = ioc_name(major, minor);
+   if ((name == NULL) || !strcmp(name, K_NODEV))
       return (buf);
 
    return (name);
@@ -177,22 +179,19 @@ char *get_devname(unsigned int major, unsigned int minor, int pretty)
  ***************************************************************************
 */
 int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
-	       struct file_hdr *file_hdr, int reset, long interval)
+	       int reset, long interval)
 {
    unsigned long file_interval, entry;
    static unsigned long long last_uptime = 0;
-   int min, max, pt1, pt2, cpu_nr;
+   int min, max, pt1, pt2;
    double f;
 
+   /* uptime is expressed in jiffies (basis of 1 processor) */
    if (!last_uptime || reset)
       last_uptime = uptime_ref;
 
-   cpu_nr = file_hdr->sa_proc;
-   if (!cpu_nr)
-      cpu_nr = 1;
-
    /* Interval cannot be greater than 0xffffffff here */
-   f = (((double) ((uptime - last_uptime) & 0xffffffff)) / cpu_nr) / HZ;
+   f = ((double) ((uptime - last_uptime) & 0xffffffff)) / HZ;
    file_interval = (unsigned long) f;
    if ((f * 10) - (file_interval * 10) >= 5)
       file_interval++; /* Rounding to correct value */
@@ -212,7 +211,7 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
     *       (Pn * Iu) or (P'n * Iu) belongs to In
     * with  Pn = En / Iu and P'n = En / Iu + 1
     */
-   f = (((double) ((uptime - uptime_ref) & 0xffffffff)) / cpu_nr) / HZ;
+   f = ((double) ((uptime - uptime_ref) & 0xffffffff)) / HZ;
    entry = (unsigned long) f;
    if ((f * 10) - (entry * 10) >= 5)
       entry++;
@@ -254,16 +253,16 @@ int decode_timestamp(char timestamp[], struct tstamp *tse)
  * Compare two time stamps
  ***************************************************************************
  */
-int datecmp(struct tm *loc_time, struct tstamp *tse)
+int datecmp(struct tm *rectime, struct tstamp *tse)
 {
-   if (loc_time->tm_hour == tse->tm_hour) {
-      if (loc_time->tm_min == tse->tm_min)
-	return (loc_time->tm_sec - tse->tm_sec);
+   if (rectime->tm_hour == tse->tm_hour) {
+      if (rectime->tm_min == tse->tm_min)
+	return (rectime->tm_sec - tse->tm_sec);
       else
-	return (loc_time->tm_min - tse->tm_min);
+	return (rectime->tm_min - tse->tm_min);
    }
    else
-     return (loc_time->tm_hour - tse->tm_hour);
+     return (rectime->tm_hour - tse->tm_hour);
 }
 
 
@@ -291,11 +290,11 @@ int parse_timestamp(char *argv[], int *opt, struct tstamp *tse,
  * Set current daily datafile
  ***************************************************************************
  */
-void set_default_file(struct tm *loc_time, char *datafile)
+void set_default_file(struct tm *rectime, char *datafile)
 {
-   get_time(loc_time);
+   get_time(rectime);
    snprintf(datafile, MAX_FILE_LEN,
-	    "%s/sa%02d", SA_DIR, loc_time->tm_mday);
+	    "%s/sa%02d", SA_DIR, rectime->tm_mday);
    datafile[MAX_FILE_LEN - 1] = '\0';
 }
 
@@ -313,30 +312,12 @@ void get_itv_value(struct file_stats *file_stats_curr,
 		   unsigned long long *itv, unsigned long long *g_itv)
 {
    /* Interval value in jiffies */
-   if (!file_stats_prev->uptime)
-      /*
-       * Stats from boot time to be displayed: only in this case we admit
-       * that the interval (which is unsigned long long) may be greater
-       * than 0xffffffff, else it was an overflow.
-       */
-      *g_itv = file_stats_curr->uptime;
-   else
-      *g_itv = (file_stats_curr->uptime - file_stats_prev->uptime)
-	 & 0xffffffff;
+   *g_itv = get_interval(file_stats_prev->uptime,
+			 file_stats_curr->uptime);
 
-   if (!(*g_itv))	/* Paranoia checking */
-      *g_itv = 1;
-
-   if (nr_proc > 1) {
-      if (!file_stats_prev->uptime0)
-	 *itv = file_stats_curr->uptime0;
-      else
-	 *itv = (file_stats_curr->uptime0 - file_stats_prev->uptime0)
-	    & 0xffffffff;
-
-      if (!(*itv))
-	 *itv = 1;
-   }
+   if (nr_proc > 1)
+      *itv = get_interval(file_stats_prev->uptime0,
+			  file_stats_curr->uptime0);
    else
       *itv = *g_itv;
 }
@@ -344,31 +325,31 @@ void get_itv_value(struct file_stats *file_stats_curr,
 
 /*
  ***************************************************************************
- * Fill loc_time structure according to data saved in file header
+ * Fill rectime structure according to data saved in file header
  ***************************************************************************
  */
-void set_hdr_loc_time(unsigned int flags, struct tm *loc_time,
+void set_hdr_rectime(unsigned int flags, struct tm *rectime,
 		      struct file_hdr *file_hdr)
 {
    struct tm *loc_t;
 
    if (PRINT_TRUE_TIME(flags)) {
       /* Get local time */
-      get_time(loc_time);
+      get_time(rectime);
 
-      loc_time->tm_mday = file_hdr->sa_day;
-      loc_time->tm_mon  = file_hdr->sa_month;
-      loc_time->tm_year = file_hdr->sa_year;
+      rectime->tm_mday = file_hdr->sa_day;
+      rectime->tm_mon  = file_hdr->sa_month;
+      rectime->tm_year = file_hdr->sa_year;
       /*
        * Call mktime() to set DST (Daylight Saving Time) flag.
        * Has anyone a better way to do it?
        */
-      loc_time->tm_hour = loc_time->tm_min = loc_time->tm_sec = 0;
-      mktime(loc_time);
+      rectime->tm_hour = rectime->tm_min = rectime->tm_sec = 0;
+      mktime(rectime);
    }
    else {
-      loc_t = localtime(&(file_hdr->sa_ust_time));
-      *loc_time = *loc_t;
+      loc_t = localtime((const time_t *) &(file_hdr->sa_ust_time));
+      *rectime = *loc_t;
    }
 }
 
@@ -378,13 +359,13 @@ void set_hdr_loc_time(unsigned int flags, struct tm *loc_time,
  * Print report header
  ***************************************************************************
  */
-void print_report_hdr(unsigned int flags, struct tm *loc_time,
+void print_report_hdr(unsigned int flags, struct tm *rectime,
 		      struct file_hdr *file_hdr)
 {
 
-   set_hdr_loc_time(flags, loc_time, file_hdr);
+   set_hdr_rectime(flags, rectime, file_hdr);
 
-   print_gal_header(loc_time, file_hdr->sa_sysname, file_hdr->sa_release,
+   print_gal_header(rectime, file_hdr->sa_sysname, file_hdr->sa_release,
 		    file_hdr->sa_nodename);
 }
 
@@ -396,8 +377,8 @@ void print_report_hdr(unsigned int flags, struct tm *loc_time,
  ***************************************************************************
  */
 unsigned int check_iface_reg(struct file_hdr *file_hdr,
-			     struct stats_net_dev *st_net_dev[], short curr,
-			     short ref, unsigned int pos)
+			     struct stats_net_dev *st_net_dev[], int curr,
+			     int ref, unsigned int pos)
 {
    struct stats_net_dev *st_net_dev_i, *st_net_dev_j;
    unsigned int index = 0;
@@ -502,7 +483,7 @@ unsigned int check_iface_reg(struct file_hdr *file_hdr,
  ***************************************************************************
  */
 int check_disk_reg(struct file_hdr *file_hdr, struct disk_stats *st_disk[],
-		   short curr, short ref, int pos)
+		   int curr, int ref, int pos)
 {
    struct disk_stats *st_disk_i, *st_disk_j;
    int index = 0;
@@ -661,7 +642,7 @@ void prep_file_for_reading(int *ifd, char *dfile, struct file_hdr *file_hdr,
  ***************************************************************************
  */
 int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
-		  unsigned int *flags, short *dis_hdr, int caller,
+		  unsigned int *flags, int caller,
 		  unsigned char irq_bitmap[], unsigned char cpu_bitmap[])
 {
    int i;
@@ -683,34 +664,27 @@ int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
 	 break;
        case 'B':
 	 *actflag |= A_PAGE;
-	 (*dis_hdr)++;
 	 break;
        case 'b':
 	 *actflag |= A_IO;
-	 (*dis_hdr)++;
 	 break;
        case 'c':
 	 *actflag |= A_PROC;
-	 (*dis_hdr)++;
 	 break;
        case 'd':
 	 *actflag |= A_DISK;
-	 (*dis_hdr)++;
 	 break;
        case 'p':
 	 *flags |= S_F_DEV_PRETTY;
 	 break;
        case 'q':
 	 *actflag |= A_QUEUE;
-	 (*dis_hdr)++;
 	 break;
        case 'r':
 	 *actflag |= A_MEM_AMT;
-	 (*dis_hdr)++;
 	 break;
        case 'R':
 	 *actflag |= A_MEMORY;
-	 (*dis_hdr)++;
 	 break;
        case 't':
 	 if (caller == C_SAR)
@@ -720,23 +694,18 @@ int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
 	 break;
        case 'u':
 	 *actflag |= A_CPU;
-	 (*dis_hdr)++;
 	 break;
        case 'v':
 	 *actflag |= A_KTABLES;
-	 (*dis_hdr)++;
 	 break;
        case 'w':
 	 *actflag |= A_CTXSW;
-	 (*dis_hdr)++;
 	 break;
        case 'W':
 	 *actflag |= A_SWAP;
-	 (*dis_hdr)++;
 	 break;
        case 'y':
 	 *actflag |= A_SERIAL;
-	 (*dis_hdr)++;
 	 break;
        case 'V':
 	 print_version();
@@ -754,8 +723,7 @@ int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
  * Parse sar "-n" option
  ***************************************************************************
  */
-int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
-		    short *dis_hdr)
+int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag)
 {
    if (!strcmp(argv[*opt], K_DEV))
       *actflag |= A_NET_DEV;
@@ -767,10 +735,8 @@ int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
       *actflag |= A_NET_NFS;
    else if (!strcmp(argv[*opt], K_NFSD))
       *actflag |= A_NET_NFSD;
-   else if (!strcmp(argv[*opt], K_ALL)) {
+   else if (!strcmp(argv[*opt], K_ALL))
       *actflag |= A_NET_DEV + A_NET_EDEV + A_NET_SOCK + A_NET_NFS + A_NET_NFSD;
-      *dis_hdr = 9;
-   }
    else
       return 1;
 
@@ -785,7 +751,7 @@ int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
  ***************************************************************************
  */
 int parse_sar_I_opt(char *argv[], int *opt, unsigned int *actflag,
-		    short *dis_hdr, unsigned char irq_bitmap[])
+		    unsigned char irq_bitmap[])
 {
    int i;
 
@@ -794,16 +760,13 @@ int parse_sar_I_opt(char *argv[], int *opt, unsigned int *actflag,
    else {
       *actflag |= A_ONE_IRQ;
       if (!strcmp(argv[*opt], K_ALL)) {
-	 *dis_hdr = 9;
 	 /* Set bit for the first 16 irq */
 	 irq_bitmap[0] = 0xff;
 	 irq_bitmap[1] = 0xff;
       }
-      else if (!strcmp(argv[*opt], K_XALL)) {
-	 *dis_hdr = 9;
+      else if (!strcmp(argv[*opt], K_XALL))
 	 /* Set every bit */
 	 init_bitmap(irq_bitmap, ~0, NR_IRQS);
-      }
       else {
 	 /*
 	  * Get irq number.
@@ -827,15 +790,13 @@ int parse_sar_I_opt(char *argv[], int *opt, unsigned int *actflag,
  ***************************************************************************
  */
 int parse_sa_P_opt(char *argv[], int *opt, unsigned int *flags,
-		   short *dis_hdr, unsigned char cpu_bitmap[])
+		   unsigned char cpu_bitmap[])
 {
    int i;
 
    if (argv[++(*opt)]) {
       *flags |= S_F_PER_PROC;
-      (*dis_hdr)++;
       if (!strcmp(argv[*opt], K_ALL)) {
-	 *dis_hdr = 9;
 	 /*
 	  * Set bit for every processor.
 	  * We still don't know if we are going to read stats
