@@ -1,6 +1,6 @@
 /*
  * sadc: system activity data collector
- * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2007 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -52,7 +52,6 @@ unsigned int serial_nr = 0, iface_nr = 0, irqcpu_nr = 0, disk_nr = 0;
 int pid_idx = 0, pid_nr = 0;
 unsigned int sadc_actflag;
 long interval = 0;
-int kb_shift = 0;
 
 struct file_hdr file_hdr;
 struct file_stats file_stats;
@@ -143,8 +142,8 @@ void p_write_error(void)
 void init_dk_drive_stat(void)
 {
    file_stats.dk_drive = 0;
-   file_stats.dk_drive_rio  = file_stats.dk_drive_wio  = 0;
-   file_stats.dk_drive_rblk = file_stats.dk_drive_wblk = 0;
+   file_stats.dk_drive_rio = file_stats.dk_drive_rblk = 0;
+   file_stats.dk_drive_wio = file_stats.dk_drive_wblk = 0;
 }
 
 
@@ -324,7 +323,7 @@ unsigned int get_irqcpu_nb(unsigned int max_nr_irqcpu)
  */
 void sa_sys_init(unsigned int *flags)
 {
-   /* How many processors on this machine ? */
+   /* How many processors on this machine? */
    if ((cpu_nr = get_cpu_nr(NR_CPUS)) > 0)
       SREALLOC(st_cpu, struct stats_one_cpu, STATS_ONE_CPU_SIZE * cpu_nr);
 
@@ -413,22 +412,22 @@ int ask_for_flock(int fd, unsigned int *flags, int fatal)
 void setup_file_hdr(int fd, size_t *file_stats_size)
 {
    int nb;
-   struct tm loc_time;
+   struct tm rectime;
    struct utsname header;
 
    /* First reset the structure */
    memset(&file_hdr, 0, FILE_HDR_SIZE);
 
    /* Then get current date */
-   file_hdr.sa_ust_time = get_localtime(&loc_time);
+   file_hdr.sa_ust_time = get_time(&rectime);
 
    /* Ok, now fill the header */
    file_hdr.sa_actflag = sadc_actflag;
    file_hdr.sa_magic = SA_MAGIC;
    file_hdr.sa_st_size = FILE_STATS_SIZE;
-   file_hdr.sa_day = loc_time.tm_mday;
-   file_hdr.sa_month = loc_time.tm_mon;
-   file_hdr.sa_year = loc_time.tm_year;
+   file_hdr.sa_day = rectime.tm_mday;
+   file_hdr.sa_month = rectime.tm_mon;
+   file_hdr.sa_year = rectime.tm_year;
    file_hdr.sa_sizeof_long = sizeof(long);
    file_hdr.sa_proc = cpu_nr;
    file_hdr.sa_nr_pid = pid_nr;
@@ -471,23 +470,23 @@ void setup_file_hdr(int fd, size_t *file_stats_size)
 void write_dummy_record(int ofd, size_t file_stats_size, unsigned int *flags)
 {
    int nb;
-   struct tm loc_time;
+   struct tm rectime;
 
    /* Check if file is locked */
    if (!FILE_LOCKED(*flags))
       ask_for_flock(ofd, flags, FATAL);
 
-   /* Reset the structure (not compulsory, but a bit cleaner */
+   /* Reset the structure (not compulsory, but a bit cleaner) */
    memset(&file_stats, 0, FILE_STATS_SIZE);
 
    file_stats.record_type = R_DUMMY;
 
    /* Save time */
-   file_stats.ust_time = get_localtime(&loc_time);
+   file_stats.ust_time = get_time(&rectime);
 
-   file_stats.hour   = loc_time.tm_hour;
-   file_stats.minute = loc_time.tm_min;
-   file_stats.second = loc_time.tm_sec;
+   file_stats.hour   = rectime.tm_hour;
+   file_stats.minute = rectime.tm_min;
+   file_stats.second = rectime.tm_sec;
 
    /* Write record now */
    if ((nb = write(ofd, &file_stats, file_stats_size)) != file_stats_size)
@@ -739,11 +738,6 @@ void read_proc_stat(unsigned int flags)
 	  * Compute the uptime of the system in jiffies (1/100ths of a second
 	  * if HZ=100).
 	  * Machine uptime is multiplied by the number of processors here.
-	  * Note that overflow is not so far away: ULONG_MAX is 4294967295 on
-	  * 32 bit systems. Overflow happens when machine uptime is:
-	  * 497 days on a monoprocessor machine,
-	  * 248 days on a bi processor,
-	  * 124 days on a quad processor...
 	  */
 	 file_stats.uptime = file_stats.cpu_user   + file_stats.cpu_nice +
 	                     file_stats.cpu_system + file_stats.cpu_idle +
@@ -774,13 +768,13 @@ void read_proc_stat(unsigned int flags)
 	       st_cpu_i->per_cpu_iowait = cc_iowait;
 	       st_cpu_i->per_cpu_steal  = cc_steal;
 	    }
-	    /* else:
-	     * Additional CPUs have been dynamically registered in /proc/stat.
-	     * sar won't crash, but the CPU stats might be false...
-	     */
+	    /* else additional CPUs have been dynamically registered in /proc/stat */
 	
-	    if (!proc_nb)
-	       /* Compute uptime reduced to one proc using proc#0 */
+	    if (!proc_nb && !file_stats.uptime0)
+	       /*
+		* Compute uptime reduced to one proc using proc#0.
+		* Done if /proc/uptime was unavailable.
+		*/
 	       file_stats.uptime0 = cc_user + cc_nice + cc_system +
 	       			    cc_idle + cc_iowait + cc_steal;
 	 }
@@ -881,9 +875,9 @@ void read_proc_stat(unsigned int flags)
 
 	       if (dsk < disk_nr) {
 		  st_disk_i = st_disk + dsk++;
-		  st_disk_i->major = v_major;
-		  st_disk_i->minor = v_index;
-		  st_disk_i->nr_ios = v_tmp[0];
+		  st_disk_i->major   = v_major;
+		  st_disk_i->minor   = v_index;
+		  st_disk_i->nr_ios  = v_tmp[0];
 		  st_disk_i->rd_sect = v_tmp[2];
 		  st_disk_i->wr_sect = v_tmp[4];
 	       }
@@ -944,43 +938,20 @@ void read_proc_loadavg(void)
  */
 void read_proc_meminfo(void)
 {
-   FILE *fp;
-   static char line[128];
-
-   if ((fp = fopen(MEMINFO, "r")) == NULL)
+   struct meminf st_mem;
+   
+   memset(&st_mem, 0, sizeof(struct meminf));
+   
+   if (readp_meminfo(&st_mem))
       return;
-
-   while (fgets(line, 128, fp) != NULL) {
-
-      if (!strncmp(line, "MemTotal:", 9))
-	 /* Read the total amount of memory in kB */
-	 sscanf(line + 9, "%lu", &(file_stats.tlmkb));
-      else if (!strncmp(line, "MemFree:", 8))
-	 /* Read the amount of free memory in kB */
-	 sscanf(line + 8, "%lu", &(file_stats.frmkb));
-
-      else if (!strncmp(line, "Buffers:", 8))
-	 /* Read the amount of buffered memory in kB */
-	 sscanf(line + 8, "%lu", &(file_stats.bufkb));
-
-      else if (!strncmp(line, "Cached:", 7))
-	 /* Read the amount of cached memory in kB */
-	 sscanf(line + 7, "%lu", &(file_stats.camkb));
-
-      else if (!strncmp(line, "SwapCached:", 11))
-	 /* Read the amount of cached swap in kB */
-	 sscanf(line + 11, "%lu", &(file_stats.caskb));
-
-      else if (!strncmp(line, "SwapTotal:", 10))
-	 /* Read the total amount of swap memory in kB */
-	 sscanf(line + 10, "%lu", &(file_stats.tlskb));
-
-      else if (!strncmp(line, "SwapFree:", 9))
-	 /* Read the amount of free swap memory in kB */
-	 sscanf(line + 9, "%lu", &(file_stats.frskb));
-   }
-
-   fclose(fp);
+   
+   file_stats.tlmkb = st_mem.tlmkb;
+   file_stats.frmkb = st_mem.frmkb;
+   file_stats.bufkb = st_mem.bufkb;
+   file_stats.camkb = st_mem.camkb;
+   file_stats.caskb = st_mem.caskb;
+   file_stats.tlskb = st_mem.tlskb;
+   file_stats.frskb = st_mem.frskb;
 }
 
 
@@ -1469,20 +1440,20 @@ void read_diskstats_stat(void)
 	       /* Unused device: ignore it */
 	       continue;
 	    st_disk_i = st_disk + dsk++;
-	    st_disk_i->major = major;
-	    st_disk_i->minor = minor;
-	    st_disk_i->nr_ios = rd_ios + wr_ios;
-	    st_disk_i->rd_sect = rd_sec;
-	    st_disk_i->wr_sect = wr_sec;
-	    st_disk_i->rd_ticks = rd_ticks;
-	    st_disk_i->wr_ticks = wr_ticks;
+	    st_disk_i->major     = major;
+	    st_disk_i->minor     = minor;
+	    st_disk_i->nr_ios    = rd_ios + wr_ios;
+	    st_disk_i->rd_sect   = rd_sec;
+	    st_disk_i->wr_sect   = wr_sec;
+	    st_disk_i->rd_ticks  = rd_ticks;
+	    st_disk_i->wr_ticks  = wr_ticks;
 	    st_disk_i->tot_ticks = tot_ticks;
-	    st_disk_i->rq_ticks = rq_ticks;
+	    st_disk_i->rq_ticks  = rq_ticks;
 	
-	    file_stats.dk_drive += rd_ios + wr_ios;
-	    file_stats.dk_drive_rio += rd_ios;
+	    file_stats.dk_drive      += rd_ios + wr_ios;
+	    file_stats.dk_drive_rio  += rd_ios;
 	    file_stats.dk_drive_rblk += (unsigned int) rd_sec;
-	    file_stats.dk_drive_wio += wr_ios;
+	    file_stats.dk_drive_wio  += wr_ios;
 	    file_stats.dk_drive_wblk += (unsigned int) wr_sec;
 	 }
       }
@@ -1530,20 +1501,20 @@ void read_ppartitions_stat(void)
 	    if (ioc_iswhole(major, minor)) {
 	       /* OK: it's a device and not a partition */
 	       st_disk_i = st_disk + dsk++;
-	       st_disk_i->major = major;
-	       st_disk_i->minor = minor;
-	       st_disk_i->nr_ios = rd_ios + wr_ios;
-	       st_disk_i->rd_sect = rd_sec;
-	       st_disk_i->wr_sect = wr_sec;
-	       st_disk_i->rd_ticks = rd_ticks;
-	       st_disk_i->wr_ticks = wr_ticks;
+	       st_disk_i->major     = major;
+	       st_disk_i->minor     = minor;
+	       st_disk_i->nr_ios    = rd_ios + wr_ios;
+	       st_disk_i->rd_sect   = rd_sec;
+	       st_disk_i->wr_sect   = wr_sec;
+	       st_disk_i->rd_ticks  = rd_ticks;
+	       st_disk_i->wr_ticks  = wr_ticks;
 	       st_disk_i->tot_ticks = tot_ticks;
-	       st_disk_i->rq_ticks = rq_ticks;
+	       st_disk_i->rq_ticks  = rq_ticks;
 	
-	       file_stats.dk_drive += rd_ios + wr_ios;
-	       file_stats.dk_drive_rio += rd_ios;
+	       file_stats.dk_drive      += rd_ios + wr_ios;
+	       file_stats.dk_drive_rio  += rd_ios;
 	       file_stats.dk_drive_rblk += (unsigned int) rd_sec;
-	       file_stats.dk_drive_wio += wr_ios;
+	       file_stats.dk_drive_wio  += wr_ios;
 	       file_stats.dk_drive_wblk += (unsigned int) wr_sec;
 	    }
 	 }
@@ -1570,7 +1541,19 @@ void read_ppartitions_stat(void)
  */
 void read_stats(unsigned int *flags)
 {
+   /*
+    * Init uptime0. So if /proc/uptime cannot fill it,
+    * this will be done by /proc/stat.
+    * If cpu_nr = 1, force /proc/stat to fill it.
+    * If cpu_nr = 0, uptime0 and uptime are equal.
+    * NB: uptime0 is always filled.
+    */
+   file_stats.uptime0 = 0;
+   if (cpu_nr > 1)
+      readp_uptime(&(file_stats.uptime0));
    read_proc_stat(*flags);
+   if (!cpu_nr)
+      file_stats.uptime0 = file_stats.uptime;
    read_proc_meminfo();
    read_proc_loadavg();
    read_proc_vmstat();
@@ -1603,7 +1586,7 @@ void read_stats(unsigned int *flags)
  * and display them.
  ***************************************************************************
  */
-void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *loc_time,
+void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *rectime,
 		     int stdfd, int ofd, size_t file_stats_size, char ofile[],
 		     char new_ofile[])
 {
@@ -1617,6 +1600,8 @@ void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *loc_time,
        * Init file_stats structure. Every record of other structures
        * is set when reading corresponding stat file (records are set
        * to 0 if there are not enough data to fill the structure).
+       * Exception for individual CPU's structures which must not be
+       * init'ed to keep values for CPU before they were disabled.
        */
       memset(&file_stats, 0, FILE_STATS_SIZE);
 
@@ -1627,10 +1612,10 @@ void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *loc_time,
 	 file_stats.record_type = R_STATS;
 
       /* Save time */
-      file_stats.ust_time = get_localtime(loc_time);
-      file_stats.hour   = loc_time->tm_hour;
-      file_stats.minute = loc_time->tm_min;
-      file_stats.second = loc_time->tm_sec;
+      file_stats.ust_time = get_time(rectime);
+      file_stats.hour   = rectime->tm_hour;
+      file_stats.minute = rectime->tm_min;
+      file_stats.second = rectime->tm_sec;
 
       /* Read then write stats */
       read_stats(flags);
@@ -1687,10 +1672,7 @@ void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *loc_time,
       /* Rotate activity file if necessary */
       if (WANT_SA_ROTAT(*flags)) {
 	 /* The user specified '-' as the filename to use */
-	 get_localtime(loc_time);
-	 snprintf(new_ofile, MAX_FILE_LEN,
-		  "%s/sa%02d", SA_DIR, loc_time->tm_mday);
-	 new_ofile[MAX_FILE_LEN - 1] = '\0';
+	 set_default_file(rectime, new_ofile);
 
 	 if (strcmp(ofile, new_ofile))
 	    do_sa_rotat = TRUE;
@@ -1715,7 +1697,7 @@ int main(int argc, char **argv)
    char ofile[MAX_FILE_LEN];
    char new_ofile[MAX_FILE_LEN];
    unsigned int flags = 0;
-   struct tm loc_time;
+   struct tm rectime;
    int stdfd = 0, ofd = -1;
    long count = 0;
    /*
@@ -1728,7 +1710,7 @@ int main(int argc, char **argv)
    size_t file_stats_size = FILE_STATS_SIZE;
 
    /* Compute page shift in kB */
-   kb_shift = get_kb_shift();
+   get_kb_shift();
 
    ofile[0] = new_ofile[0] = '\0';
 
@@ -1793,10 +1775,7 @@ int main(int argc, char **argv)
 	    stdfd = -1;	/* Don't write to STDOUT */
 	    if (!strcmp(argv[opt], "-")) {
 	       /* File name set to '-' */
-	       get_localtime(&loc_time);
-	       snprintf(ofile, MAX_FILE_LEN,
-			"%s/sa%02d", SA_DIR, loc_time.tm_mday);
-	       ofile[MAX_FILE_LEN - 1] = '\0';
+	       set_default_file(&rectime, ofile);
 	       flags |= S_F_SA_ROTAT;
 	    }
 	    else if (!strncmp(argv[opt], "-", 1))
@@ -1860,8 +1839,8 @@ int main(int argc, char **argv)
 
    /*
     * Open output file then STDOUT. Write header for each of them.
-    * NB: Output file must be opened first because we may change
-    * the activity flag to that of the file, and the activity flag
+    * NB: Output file must be opened first, because we may change
+    * the activity flag to that of the file and the activity flag
     * written on STDOUT must be consistent.
     */
    open_ofile(&ofd, ofile, &file_stats_size, &flags);
@@ -1879,7 +1858,7 @@ int main(int argc, char **argv)
    alarm_handler(0);
 
    /* Main loop */
-   rw_sa_stat_loop(&flags, count, &loc_time, stdfd, ofd,
+   rw_sa_stat_loop(&flags, count, &rectime, stdfd, ofd,
 		   file_stats_size, ofile, new_ofile);
 
    return 0;
