@@ -1,6 +1,6 @@
 /*
  * sadf: system activity data formatter
- * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2007 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -26,11 +26,11 @@
 #include <unistd.h>
 #include <time.h>
 #include <errno.h>
-#include <sys/param.h>	/* for HZ */
 
 #include "sadf.h"
 #include "sa.h"
 #include "common.h"
+#include "ioconf.h"
 
 
 #ifdef USE_NLS
@@ -45,28 +45,26 @@
 long interval = -1, count = 0;
 unsigned int sadf_actflag = 0;
 unsigned int flags = 0;
-unsigned short format = 0;	/* Output format */
+unsigned int format = 0;	/* Output format */
 unsigned char irq_bitmap[(NR_IRQS / 8) + 1];
 unsigned char cpu_bitmap[(NR_CPUS / 8) + 1];
-int kb_shift = 0;
 
 struct file_hdr file_hdr;
-struct file_stats file_stats[DIM];
-struct stats_one_cpu *st_cpu[DIM];
-struct stats_serial *st_serial[DIM];
-struct stats_irq_cpu *st_irq_cpu[DIM];
-struct stats_net_dev *st_net_dev[DIM];
-struct disk_stats *st_disk[DIM];
+struct file_stats file_stats[3];
+struct stats_one_cpu *st_cpu[3];
+struct stats_serial *st_serial[3];
+struct stats_irq_cpu *st_irq_cpu[3];
+struct stats_net_dev *st_net_dev[3];
+struct disk_stats *st_disk[3];
 
 /* Array members of common types are always packed */
-unsigned int interrupts[DIM][NR_IRQS];
+unsigned int interrupts[3][NR_IRQS];
 
-struct tm loc_time;
+struct tm rectime, loctime;
 /* Contain the date specified by -s and -e options */
 struct tstamp tm_start, tm_end;
 char *args[MAX_ARGV_NR];
 
-
 /*
  ***************************************************************************
  * Print usage and exit
@@ -103,7 +101,7 @@ void prtab(int nr_tab)
  * printf() function modified for XML display
  ***************************************************************************
  */
-void xprintf(short nr_tab, const char *fmt, ...)
+void xprintf(int nr_tab, const char *fmt, ...)
 {
    static char buf[1024];
    va_list args;
@@ -119,26 +117,25 @@ void xprintf(short nr_tab, const char *fmt, ...)
 
 /*
  ***************************************************************************
- * Fill loc_time structure according to time data saved in file.
+ * Fill rectime structure according to time data saved in file.
  * NB: Option -t is ignored when option -p is used, since option -p
  * displays its timestamp as a long integer. This is type 'time_t',
  * which is the number of seconds since 1970 _always_ expressed in UTC.
  ***************************************************************************
 */
-void set_loc_time(short curr)
+void set_rectime(int curr)
 {
    struct tm *ltm;
 
-   /* NOTE: loc_time structure must have been init'ed before! */
-   if (PRINT_TRUE_TIME(flags) &&
-       ((format == S_O_DB_OPTION) || (format == S_O_XML_OPTION)))
-      /* '-d -t' or '-x -t'. Option -t is ignored with option -p and -D */
-      ltm = localtime(&file_stats[curr].ust_time);
-   else
-      /* '-p' od '-D' or '-d' or '-x' */
-      ltm = gmtime(&file_stats[curr].ust_time);
+   ltm = localtime((const time_t *) &file_stats[curr].ust_time);
+   loctime = *ltm;
 
-   loc_time = *ltm;
+   if (!PRINT_TRUE_TIME(flags) ||
+       ((format != S_O_DB_OPTION) && (format != S_O_XML_OPTION)))
+      /* Option -t is valid only with options -d and -x */
+      ltm = gmtime((const time_t *) &file_stats[curr].ust_time);
+
+   rectime = *ltm;
 }
 
 
@@ -147,16 +144,16 @@ void set_loc_time(short curr)
  * Set timestamp string
  ***************************************************************************
 */
-void set_timestamp(short curr, char *cur_time, int len)
+void set_timestamp(int curr, char *cur_time, int len)
 {
-   set_loc_time(curr);
+   set_rectime(curr);
 	
    /* Set cur_time date value */
    if (format == S_O_DB_OPTION) {
       if (PRINT_TRUE_TIME(flags))
-	 strftime(cur_time, len, "%Y-%m-%d %H:%M:%S", &loc_time);
+	 strftime(cur_time, len, "%Y-%m-%d %H:%M:%S", &rectime);
       else
-	 strftime(cur_time, len, "%Y-%m-%d %H:%M:%S UTC", &loc_time);
+	 strftime(cur_time, len, "%Y-%m-%d %H:%M:%S UTC", &rectime);
    }
    else if ((format == S_O_PPC_OPTION) || (format == S_O_DBD_OPTION))
       sprintf(cur_time, "%ld", file_stats[curr].ust_time);
@@ -275,7 +272,7 @@ static void render(int isdb, char *pre, int rflags, const char *pptxt,
  * making it easier for them to remain in sync and print the same data.
  ***************************************************************************
  */
-void write_mech_stats(short curr, unsigned int act,
+void write_mech_stats(int curr, unsigned int act,
 		      unsigned long dt, unsigned long long itv,
 		      unsigned long long g_itv, char *cur_time)
 {
@@ -293,7 +290,6 @@ void write_mech_stats(short curr, unsigned int act,
    snprintf(pre, 80, "%s%s%ld%s%s",
 	    file_hdr.sa_nodename, seps[isdb], dt, seps[isdb], cur_time);
 
-
    if (GET_PROC(act)) {
       /* The first one as an example */
       render(isdb,		/* db/ppc flag */
@@ -314,7 +310,6 @@ void write_mech_stats(short curr, unsigned int act,
 	     ll_s_value(fsj->context_swtch, fsi->context_swtch, itv));
    }
 
-
    if (GET_CPU(act) && wantproc) {
       render(isdb, pre,
 	     PT_NOFLAG,		/* that's zero but you know what it means */
@@ -347,12 +342,11 @@ void write_mech_stats(short curr, unsigned int act,
       render(isdb, pre, PT_NEWLIN,
 	     "all\t%%idle", NULL, NULL,
 	     NOVAL,
-	     (fsi->cpu_idle < fsj->cpu_idle)
-	     ? 0.0
-	     : ll_sp_value(fsj->cpu_idle, fsi->cpu_idle, g_itv));
+	     (fsi->cpu_idle < fsj->cpu_idle) ?
+	     0.0 :
+	     ll_sp_value(fsj->cpu_idle, fsi->cpu_idle, g_itv));
    }
 
-
    if (GET_CPU(act) && WANT_PER_PROC(flags) && file_hdr.sa_proc) {
       int i;
       unsigned long long pc_itv;
@@ -371,34 +365,51 @@ void write_mech_stats(short curr, unsigned int act,
 		   "%d",		/* db text with format char */
 		   cons(iv, i, NOVAL),	/* how we pass format args */
 		   NOVAL,
+		   !pc_itv ?
+		   0.0 :		/* CPU is offline */
 		   ll_sp_value(scj->per_cpu_user, sci->per_cpu_user, pc_itv));
 
 	    render(isdb, pre, PT_NOFLAG,
 		   "cpu%d\t%%nice", NULL, cons(iv, i, NOVAL),
 		   NOVAL,
+		   !pc_itv ?
+		   0.0 :
 		   ll_sp_value(scj->per_cpu_nice, sci->per_cpu_nice, pc_itv));
 
 	    render(isdb, pre, PT_NOFLAG,
 		   "cpu%d\t%%system", NULL, cons(iv, i, NOVAL),
 		   NOVAL,
+		   !pc_itv ?
+		   0.0 :
 		   ll_sp_value(scj->per_cpu_system, sci->per_cpu_system, pc_itv));
 
 	    render(isdb, pre, PT_NOFLAG,
 		   "cpu%d\t%%iowait", NULL, cons(iv, i, NOVAL),
 		   NOVAL,
+		   !pc_itv ?
+		   0.0 :
 		   ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv));
 
 	    render(isdb, pre, PT_NOFLAG,
 		   "cpu%d\t%%steal", NULL, cons(iv, i, NOVAL),
 		   NOVAL,
+		   !pc_itv ?
+		   0.0 :
 		   ll_sp_value(scj->per_cpu_steal, sci->per_cpu_steal, pc_itv));
 
-	    render(isdb, pre, PT_NEWLIN,
-		   "cpu%d\t%%idle", NULL, cons(iv, i, NOVAL),
-		   NOVAL,
-		   (sci->per_cpu_idle < scj->per_cpu_idle)
-		   ? 0.0
-		   : ll_sp_value(scj->per_cpu_idle, sci->per_cpu_idle, pc_itv));
+	    if (!pc_itv)
+	       /* CPU is offline */
+	       render(isdb, pre, PT_NEWLIN,
+		      "cpu%d\t%%idle", NULL, cons(iv, i, NOVAL),
+		      NOVAL,
+		      0.0);
+	    else
+	       render(isdb, pre, PT_NEWLIN,
+		      "cpu%d\t%%idle", NULL, cons(iv, i, NOVAL),
+		      NOVAL,
+		      (sci->per_cpu_idle < scj->per_cpu_idle) ?
+		      0.0 :
+		      ll_sp_value(scj->per_cpu_idle, sci->per_cpu_idle, pc_itv));
 	 }
       }
    }
@@ -621,7 +632,6 @@ void write_mech_stats(short curr, unsigned int act,
       }
    }
 
-
    /* Print values of some kernel tables */
    if (GET_KTABLES(act)) {
       render(isdb, pre, PT_USEINT,
@@ -696,12 +706,12 @@ void write_mech_stats(short curr, unsigned int act,
 		NOVAL, S_VALUE(sndj->tx_packets, sndi->tx_packets, itv));
 
 	 render(isdb, pre, PT_NOFLAG,
-		"%s\trxbyt/s", NULL, cons(sv, ifc, NULL),
-		NOVAL, S_VALUE(sndj->rx_bytes, sndi->rx_bytes, itv));
+		"%s\trxkB/s", NULL, cons(sv, ifc, NULL),
+		NOVAL, S_VALUE(sndj->rx_bytes, sndi->rx_bytes, itv) / 1024);
 
 	 render(isdb, pre, PT_NOFLAG,
-		"%s\ttxbyt/s", NULL, cons(sv, ifc, NULL),
-		NOVAL, S_VALUE(sndj->tx_bytes, sndi->tx_bytes, itv));
+		"%s\ttxkB/s", NULL, cons(sv, ifc, NULL),
+		NOVAL, S_VALUE(sndj->tx_bytes, sndi->tx_bytes, itv) / 1024);
 
 	 render(isdb, pre, PT_NOFLAG,
 		"%s\trxcmp/s", NULL, cons(sv, ifc, NULL),
@@ -793,7 +803,6 @@ void write_mech_stats(short curr, unsigned int act,
 	     "-\tip-frag", NULL, NULL, fsi->frag_inuse, DNOVAL);
    }
 
-
    /* Print load averages and queue length */
    if (GET_QUEUE(act)) {
       render(isdb, pre, PT_USEINT,
@@ -818,7 +827,7 @@ void write_mech_stats(short curr, unsigned int act,
    /* Print disk statistics */
    if (GET_DISK(act)) {
       int i, j;
-      char *name;
+      char *name = NULL;
       double tput, util, await, svctm, arqsz;
       struct disk_stats
 	 *sdi = st_disk[curr],
@@ -831,7 +840,12 @@ void write_mech_stats(short curr, unsigned int act,
 
 	 j = check_disk_reg(&file_hdr, st_disk, curr, !curr, i);
 	 sdj = st_disk[!curr] + j;
-	 name = get_devname(sdi->major, sdi->minor, USE_PRETTY_OPTION(flags));
+	 
+	 if ((USE_PRETTY_OPTION(flags)) && (sdi->major == DEVMAP_MAJOR))
+	    name = transform_devmapname(sdi->major, sdi->minor);
+	 
+	 if (!name)
+	    name = get_devname(sdi->major, sdi->minor, USE_PRETTY_OPTION(flags));
 
 	 tput = ((double) (sdi->nr_ios - sdj->nr_ios)) * HZ / itv;
 	 util = S_VALUE(sdj->tot_ticks, sdi->tot_ticks, itv);
@@ -951,14 +965,14 @@ void write_mech_stats(short curr, unsigned int act,
  * Write system statistics
  ***************************************************************************
  */
-int write_parsable_stats(short curr, unsigned int act, int reset, long *cnt,
+int write_parsable_stats(int curr, unsigned int act, int reset, long *cnt,
 			 int use_tm_start, int use_tm_end)
 {
    unsigned long long dt, itv, g_itv;
    char cur_time[26];
 
    /* Check time (1) */
-   if (!next_slice(file_stats[2].uptime, file_stats[curr].uptime, &file_hdr,
+   if (!next_slice(file_stats[2].uptime0, file_stats[curr].uptime0,
 		   reset, interval))
       /* Not close enough to desired interval */
       return 0;
@@ -967,7 +981,7 @@ int write_parsable_stats(short curr, unsigned int act, int reset, long *cnt,
    set_timestamp(curr, cur_time, 26);
 
    /* Check time (2) */
-   if (use_tm_start && (datecmp(&loc_time, &tm_start) < 0))
+   if (use_tm_start && (datecmp(&loctime, &tm_start) < 0))
      /* it's too soon... */
      return 0;
 
@@ -976,7 +990,7 @@ int write_parsable_stats(short curr, unsigned int act, int reset, long *cnt,
 		 file_hdr.sa_proc, &itv, &g_itv);
 
    /* Check time (3) */
-   if (use_tm_end && (datecmp(&loc_time, &tm_end) > 0)) {
+   if (use_tm_end && (datecmp(&loctime, &tm_end) > 0)) {
       /* It's too late... */
       *cnt = 0;
       return 0;
@@ -998,7 +1012,7 @@ int write_parsable_stats(short curr, unsigned int act, int reset, long *cnt,
  * Display XML activity records
  ***************************************************************************
  */
-void write_xml_stats(short curr, short *tab)
+void write_xml_stats(int curr, int *tab)
 {
    int i, j, k;
    unsigned long long dt, itv, g_itv;
@@ -1008,7 +1022,7 @@ void write_xml_stats(short curr, short *tab)
       *fsj = &file_stats[!curr];
 
    /* Set timestamp for current data */
-   set_loc_time(curr);
+   set_rectime(curr);
 
    /* Get interval values */
    get_itv_value(&file_stats[curr], &file_stats[!curr],
@@ -1019,7 +1033,7 @@ void write_xml_stats(short curr, short *tab)
    if ((itv % HZ) >= (HZ / 2))
       dt++;
 
-   strftime(cur_time, 64, "date=\"%Y-%m-%d\" time=\"%H:%M:%S\"", &loc_time);
+   strftime(cur_time, 64, "date=\"%Y-%m-%d\" time=\"%H:%M:%S\"", &rectime);
    xprintf(*tab, "<timestamp %s interval=\"%llu\">", cur_time, dt);
    (*tab)++;
 
@@ -1055,17 +1069,24 @@ void write_xml_stats(short curr, short *tab)
 	 /* Recalculate itv for current proc */
 	 pc_itv = get_per_cpu_interval(sci, scj);
 	
-	 xprintf(*tab, "<cpu number=\"%d\" user=\"%.2f\" nice=\"%.2f\" "
-		       "system=\"%.2f\" iowait=\"%.2f\" steal=\"%.2f\" idle=\"%.2f\"/>",
-		 i,
-		 ll_sp_value(scj->per_cpu_user, sci->per_cpu_user, pc_itv),
-		 ll_sp_value(scj->per_cpu_nice, sci->per_cpu_nice, pc_itv),
-		 ll_sp_value(scj->per_cpu_system, sci->per_cpu_system, pc_itv),
-		 ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv),
-		 ll_sp_value(scj->per_cpu_steal, sci->per_cpu_steal, pc_itv),
-		 (sci->per_cpu_idle < scj->per_cpu_idle)
-		 ? 0.0
-		 : ll_sp_value(scj->per_cpu_idle, sci->per_cpu_idle, pc_itv));
+	 if (!pc_itv)
+	    /* Current proc is offlined */
+	    xprintf(*tab, "<cpu number=\"%d\" user=\"0.00\" nice=\"0.00\" "
+		    "system=\"0.00\" iowait=\"0.00\" steal=\"0.00\" idle=\"0.00\"/>",
+		    i);
+	 else {
+	    xprintf(*tab, "<cpu number=\"%d\" user=\"%.2f\" nice=\"%.2f\" "
+		    "system=\"%.2f\" iowait=\"%.2f\" steal=\"%.2f\" idle=\"%.2f\"/>",
+		    i,
+		    ll_sp_value(scj->per_cpu_user, sci->per_cpu_user, pc_itv),
+		    ll_sp_value(scj->per_cpu_nice, sci->per_cpu_nice, pc_itv),
+		    ll_sp_value(scj->per_cpu_system, sci->per_cpu_system, pc_itv),
+		    ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv),
+		    ll_sp_value(scj->per_cpu_steal, sci->per_cpu_steal, pc_itv),
+		    (sci->per_cpu_idle < scj->per_cpu_idle)
+		    ? 0.0
+		    : ll_sp_value(scj->per_cpu_idle, sci->per_cpu_idle, pc_itv));
+	 }
       }
    }
    xprintf(--(*tab), "</cpu-load>");
@@ -1158,6 +1179,7 @@ void write_xml_stats(short curr, short *tab)
       struct disk_stats
 	 *sdi = st_disk[curr],
 	 *sdj;
+      char *name = NULL;
 
       xprintf(*tab, "<disk per=\"second\">");
       (*tab)++;
@@ -1180,11 +1202,16 @@ void write_xml_stats(short curr, short *tab)
 	    ((sdi->rd_sect - sdj->rd_sect) + (sdi->wr_sect - sdj->wr_sect)) /
 	    ((double) (sdi->nr_ios - sdj->nr_ios)) : 0.0;
 
+	 if ((USE_PRETTY_OPTION(flags)) && (sdi->major == DEVMAP_MAJOR))
+	    name = transform_devmapname(sdi->major, sdi->minor);
+	 
+	 if (!name)
+	    name = get_devname(sdi->major, sdi->minor, USE_PRETTY_OPTION(flags));
+
 	 xprintf(*tab, "<disk-device dev=\"%s\" tps=\"%.2f\" rd_sec=\"%.2f\" "
 		       "wr_sec=\"%.2f\" avgrq-sz=\"%.2f\" avgqu-sz=\"%.2f\" "
 		       "await=\"%.2f\" svctm=\"%.2f\" util-percent=\"%.2f\"/>",
-		 /* Confusion possible here between index and minor numbers */
-		 get_devname(sdi->major, sdi->minor, USE_PRETTY_OPTION(flags)),
+		 name,
 		 S_VALUE(sdj->nr_ios,  sdi->nr_ios,  itv),
 		 ll_s_value(sdj->rd_sect, sdi->rd_sect, itv),
 		 ll_s_value(sdj->wr_sect, sdi->wr_sect, itv),
@@ -1245,12 +1272,12 @@ void write_xml_stats(short curr, short *tab)
 	
 	 xprintf((*tab)++, "<net-device iface=\"%s\">",
 		 sndi->interface);
-	 xprintf(*tab, "<net-dev rxpck=\"%.2f\" txpck=\"%.2f\" rxbyt=\"%.2f\" "
-		       "txbyt=\"%.2f\" rxcmp=\"%.2f\" txcmp=\"%.2f\" rxmcst=\"%.2f\"/>",
+	 xprintf(*tab, "<net-dev rxpck=\"%.2f\" txpck=\"%.2f\" rxkB=\"%.2f\" "
+		       "txkB=\"%.2f\" rxcmp=\"%.2f\" txcmp=\"%.2f\" rxmcst=\"%.2f\"/>",
 		 S_VALUE(sndj->rx_packets, sndi->rx_packets, itv),
 		 S_VALUE(sndj->tx_packets, sndi->tx_packets, itv),
-		 S_VALUE(sndj->rx_bytes, sndi->rx_bytes, itv),
-		 S_VALUE(sndj->tx_bytes, sndi->tx_bytes, itv),
+		 S_VALUE(sndj->rx_bytes, sndi->rx_bytes, itv) / 1024,
+		 S_VALUE(sndj->tx_bytes, sndi->tx_bytes, itv) / 1024,
 		 S_VALUE(sndj->rx_compressed, sndi->rx_compressed, itv),
 		 S_VALUE(sndj->tx_compressed, sndi->tx_compressed, itv),
 		 S_VALUE(sndj->multicast, sndi->multicast, itv));
@@ -1372,14 +1399,14 @@ void write_xml_stats(short curr, short *tab)
  * Display XML restart records
  ***************************************************************************
  */
-void write_xml_restarts(short curr, short *tab)
+void write_xml_restarts(int curr, int *tab)
 {
    char cur_time[64];
 
    /* Set timestamp for current data */
-   set_loc_time(curr);
+   set_rectime(curr);
 
-   strftime(cur_time, 64, "date=\"%Y-%m-%d\" time=\"%H:%M:%S\"", &loc_time);
+   strftime(cur_time, 64, "date=\"%Y-%m-%d\" time=\"%H:%M:%S\"", &rectime);
    xprintf(*tab, "<boot %s/>", cur_time);
 }
 
@@ -1389,15 +1416,15 @@ void write_xml_restarts(short curr, short *tab)
  * Print a Linux restart message (contents of a DUMMY record)
  ***************************************************************************
  */
-void write_dummy(short curr, int use_tm_start, int use_tm_end)
+void write_dummy(int curr, int use_tm_start, int use_tm_end)
 {
    char cur_time[26];
 
    set_timestamp(curr, cur_time, 26);
 
    /* The RESTART message must be in the interval specified by -s/-e options */
-   if ((use_tm_start && (datecmp(&loc_time, &tm_start) < 0)) ||
-       (use_tm_end && (datecmp(&loc_time, &tm_end) > 0)))
+   if ((use_tm_start && (datecmp(&loctime, &tm_start) < 0)) ||
+       (use_tm_end && (datecmp(&loctime, &tm_end) > 0)))
       return;
 
    if (format == S_O_PPC_OPTION)
@@ -1418,8 +1445,9 @@ void display_file_header(char *dfile, struct file_hdr *file_hdr)
 {
    printf("File: %s (%#x)\n", dfile, file_hdr->sa_magic);
 
-   print_gal_header(localtime(&(file_hdr->sa_ust_time)), file_hdr->sa_sysname,
-		    file_hdr->sa_release, file_hdr->sa_nodename);
+   print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
+		    file_hdr->sa_sysname, file_hdr->sa_release,
+		    file_hdr->sa_nodename);
 
    printf("Activity flag: %#x\n", file_hdr->sa_actflag);
    printf("Sizeof(long): %d\n", file_hdr->sa_sizeof_long);
@@ -1436,11 +1464,11 @@ void display_file_header(char *dfile, struct file_hdr *file_hdr)
  * Display XML header and host data
  ***************************************************************************
  */
-void display_xml_header(short *tab)
+void display_xml_header(int *tab)
 {
    printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    printf("<!DOCTYPE Configure PUBLIC \"DTD v%s sysstat //EN\"\n", XML_DTD_VERSION);
-   printf("\"http://perso.wanadoo.fr/sebastien.godard/sysstat.dtd\">\n");
+   printf("\"http://perso.orange.fr/sebastien.godard/sysstat.dtd\">\n");
 
    xprintf(*tab, "<sysstat>");
    xprintf(++(*tab), "<sysdata-version>%s</sysdata-version>", XML_DTD_VERSION);
@@ -1506,7 +1534,7 @@ void copy_structures(int dest, int src)
  * Read varying part of the statistics from a daily data file
  ***************************************************************************
  */
-void read_extra_stats(short curr, int ifd)
+void read_extra_stats(int curr, int ifd)
 {
    if (file_hdr.sa_proc)
       sa_fread(ifd, st_cpu[curr],
@@ -1535,7 +1563,7 @@ void read_extra_stats(short curr, int ifd)
  * Read stats for current activity from file
  ***************************************************************************
  */
-void read_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eosaf,
+void read_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf,
 			 unsigned int act, int *reset)
 {
    unsigned char rtype;
@@ -1595,7 +1623,7 @@ void read_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eosaf
 void xml_display_loop(int ifd)
 {
    unsigned char rtype;
-   short curr, tab = 0;
+   int curr, tab = 0;
    int eosaf = TRUE;
    off_t fpos;
 
@@ -1684,7 +1712,7 @@ void xml_display_loop(int ifd)
  */
 void main_display_loop(int ifd)
 {
-   short curr = 1;
+   int curr = 1;
    unsigned int act;
    int eosaf = TRUE, reset = FALSE;
    long cnt = 1;
@@ -1707,12 +1735,12 @@ void main_display_loop(int ifd)
 	    /*
 	     Ok: previous record was not a DUMMY one. So read now the extra fields. */
 	    read_extra_stats(0, ifd);
-	    set_loc_time(0);
+	    set_rectime(0);
 	 }
       }
       while ((file_stats[0].record_type == R_DUMMY) ||
-	     (tm_start.use && (datecmp(&loc_time, &tm_start) < 0)) ||
-	     (tm_end.use && (datecmp(&loc_time, &tm_end) >=0)));
+	     (tm_start.use && (datecmp(&loctime, &tm_start) < 0)) ||
+	     (tm_end.use && (datecmp(&loctime, &tm_end) >=0)));
 
       /* Save the first stats collected. Will be used to compute the average */
       copy_structures(2, 0);
@@ -1785,8 +1813,7 @@ void read_stats_from_file(char dfile[])
    /* Perform required allocations */
    allocate_structures(USE_SA_FILE);
 
-   /* Print report header if needed */
-   print_report_hdr(format, flags, &loc_time, &file_hdr);
+   set_hdr_rectime(flags, &rectime, &file_hdr);
 
    if (format == S_O_XML_OPTION)
      xml_display_loop(ifd);
@@ -1807,10 +1834,12 @@ int main(int argc, char **argv)
    int opt = 1, sar_options = 0;
    int i;
    char dfile[MAX_FILE_LEN];
-   short dum;
+
+   /* Get HZ */
+   get_HZ();
 
    /* Compute page shift in kB */
-   kb_shift = get_kb_shift();
+   get_kb_shift();
 
    dfile[0] = '\0';
 
@@ -1829,7 +1858,7 @@ int main(int argc, char **argv)
 
       if (!strcmp(argv[opt], "-I")) {
 	 if (argv[++opt] && sar_options) {
-	    if (parse_sar_I_opt(argv, &opt, &sadf_actflag, &dum,
+	    if (parse_sar_I_opt(argv, &opt, &sadf_actflag,
 				irq_bitmap))
 	       usage(argv[0]);
 	 }
@@ -1838,7 +1867,7 @@ int main(int argc, char **argv)
       }
 
       else if (!strcmp(argv[opt], "-P")) {
-	 if (parse_sa_P_opt(argv, &opt, &flags, &dum, cpu_bitmap))
+	 if (parse_sa_P_opt(argv, &opt, &flags, cpu_bitmap))
 	    usage(argv[0]);
       }
 
@@ -1862,7 +1891,7 @@ int main(int argc, char **argv)
       else if (!strcmp(argv[opt], "-n")) {
 	 if (argv[++opt] && sar_options) {
 	    /* Parse sar's option -n */
-	    if (parse_sar_n_opt(argv, &opt, &sadf_actflag, &dum))
+	    if (parse_sar_n_opt(argv, &opt, &sadf_actflag))
 	       usage(argv[0]);
 	 }
 	 else
@@ -1872,7 +1901,7 @@ int main(int argc, char **argv)
       else if (!strncmp(argv[opt], "-", 1)) {
 	 /* Other options not previously tested */
 	 if (sar_options) {
-	    if (parse_sar_opt(argv, opt, &sadf_actflag, &flags, &dum, C_SADF,
+	    if (parse_sar_opt(argv, opt, &sadf_actflag, &flags, C_SADF,
 			      irq_bitmap, cpu_bitmap))
 	       usage(argv[0]);
 	 }
@@ -1926,10 +1955,7 @@ int main(int argc, char **argv)
 	 if (!dfile[0]) {
 	    if (!strcmp(argv[opt], "-")) {
 	       /* File name set to '-' */
-	       get_localtime(&loc_time);
-	       snprintf(dfile, MAX_FILE_LEN,
-			"%s/sa%02d", SA_DIR, loc_time.tm_mday);
-	       dfile[MAX_FILE_LEN - 1] = '\0';
+	       set_default_file(&rectime, dfile);
 	       opt++;
 	    }
 	    else if (!strncmp(argv[opt], "-", 1))
@@ -1969,12 +1995,8 @@ int main(int argc, char **argv)
    }
 
    /* sadf reads current daily data file by default */
-   if (!dfile[0]) {
-      get_localtime(&loc_time);
-      snprintf(dfile, MAX_FILE_LEN,
-	       "%s/sa%02d", SA_DIR, loc_time.tm_mday);
-      dfile[MAX_FILE_LEN - 1] = '\0';
-   }
+   if (!dfile[0])
+      set_default_file(&rectime, dfile);
 
    /*
     * Display all the contents of the daily data file if the count parameter
