@@ -1,6 +1,6 @@
 /*
  * sar: report system activity
- * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2007 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -25,10 +25,10 @@
 #include <unistd.h>
 #include <time.h>
 #include <errno.h>
-#include <sys/param.h>	/* for HZ */
 
 #include "sa.h"
 #include "common.h"
+#include "ioconf.h"
 
 
 #ifdef USE_NLS
@@ -44,30 +44,26 @@ unsigned int sar_actflag = 0;
 unsigned int flags = 0;
 unsigned char irq_bitmap[(NR_IRQS / 8) + 1];
 unsigned char cpu_bitmap[(NR_CPUS / 8) + 1];
-int kb_shift = 0;
 
 struct stats_sum asum;
 struct file_hdr file_hdr;
-struct file_stats file_stats[DIM];
-struct stats_one_cpu *st_cpu[DIM] = {NULL, NULL, NULL};
-struct stats_serial *st_serial[DIM] = {NULL, NULL, NULL};
-struct stats_irq_cpu *st_irq_cpu[DIM] = {NULL, NULL, NULL};
-struct stats_net_dev *st_net_dev[DIM] = {NULL, NULL, NULL};
-struct disk_stats *st_disk[DIM] = {NULL, NULL, NULL};
+struct file_stats file_stats[3];
+struct stats_one_cpu *st_cpu[3] = {NULL, NULL, NULL};
+struct stats_serial *st_serial[3] = {NULL, NULL, NULL};
+struct stats_irq_cpu *st_irq_cpu[3] = {NULL, NULL, NULL};
+struct stats_net_dev *st_net_dev[3] = {NULL, NULL, NULL};
+struct disk_stats *st_disk[3] = {NULL, NULL, NULL};
 
 /* Array members of common types are always packed */
-unsigned int interrupts[DIM][NR_IRQS];
-/* Structures are aligned but also padded. Thus array members are packed */
-struct pid_stats *pid_stats[DIM][MAX_PID_NR];
+unsigned int interrupts[3][NR_IRQS];
+struct pid_stats *pid_stats[3][MAX_PID_NR];
 
-struct tm loc_time;
+struct tm rectime;
 /* Contain the date specified by -s and -e options */
 struct tstamp tm_start, tm_end;
-short dis_hdr = -1;
 int pid_nr = 0;
 char *args[MAX_ARGV_NR];
 
-
 /*
  ***************************************************************************
  * Print usage and exit
@@ -101,7 +97,7 @@ void init_all_stats(void)
    init_stats(file_stats, interrupts);
    memset(&asum, 0, STATS_SUM_SIZE);
 
-   for (i = 0; i < DIM; i++)
+   for (i = 0; i < 3; i++)
       pid_stats[i][0] = NULL;
 }
 
@@ -130,7 +126,7 @@ void salloc_pid(int nr_pid)
 {
    int i, pid;
 
-   for (i = 0; i < DIM; i++) {
+   for (i = 0; i < 3; i++) {
       if (pid_stats[i][0])
 	 free(pid_stats[i][0]);
       if ((pid_stats[i][0] = (struct pid_stats *) malloc(PID_STATS_SIZE * nr_pid)) == NULL) {
@@ -224,26 +220,76 @@ void prep_smp_option(unsigned int cpu_nr)
 
 /*
  ***************************************************************************
- * Fill loc_time structure according to time data saved in current
+ * Fill rectime structure according to time data saved in current
  * structure.
  ***************************************************************************
 */
-void set_loc_time(short curr)
+void set_rectime(int curr)
 {
    struct tm *ltm;
 
-   /* NOTE: loc_time structure must have been init'ed before! */
-
    /* Check if option -t was specified on the command line */
    if (PRINT_TRUE_TIME(flags)) {
       /* -t */
-      loc_time.tm_hour = file_stats[curr].hour;
-      loc_time.tm_min  = file_stats[curr].minute;
-      loc_time.tm_sec  = file_stats[curr].second;
+      rectime.tm_hour = file_stats[curr].hour;
+      rectime.tm_min  = file_stats[curr].minute;
+      rectime.tm_sec  = file_stats[curr].second;
    }
    else {
-      ltm = localtime(&file_stats[curr].ust_time);
-      loc_time = *ltm;
+      ltm = localtime((const time_t *) &file_stats[curr].ust_time);
+      rectime = *ltm;
+   }
+}
+
+/*
+ ***************************************************************************
+ * Count number of bits set in array.
+ * @ptr: pointer to array
+ * @size: size of array in bytes
+ ***************************************************************************
+*/
+int count_bits(void *ptr, int size)
+{
+   int nr = 0, i, k;
+   char *p;
+
+   p = ptr;
+   for (i = 0; i < size; i++, p++) {
+      k = 0x80;
+      while (k) {
+	 if ((*p) & k)
+	    nr++;
+	 k >>= 1;
+      }
+   }
+
+   return nr;
+}
+
+
+/*
+ ***************************************************************************
+ * Determine if a stat line header has to be displayed
+ ***************************************************************************
+*/
+void check_line_hdr(int *dis_hdr)
+{
+   int nr_opt;
+
+   /* Get number of options entered on the command line */
+   nr_opt = count_bits(&sar_actflag, sizeof(unsigned int));
+
+   if ((nr_opt > 1) ||
+       ((GET_NET_DEV(sar_actflag) || GET_NET_EDEV(sar_actflag)) && (file_hdr.sa_iface > 1)) ||
+       (GET_DISK(sar_actflag) && (file_hdr.sa_nr_disk > 1)))
+      *dis_hdr = 1;
+   else if (GET_ONE_IRQ(sar_actflag)) {
+      if (count_bits(irq_bitmap, sizeof(irq_bitmap)) > 1)
+	 *dis_hdr = 1;
+   }
+   else if ((GET_IRQ(sar_actflag) || GET_CPU(sar_actflag)) && WANT_PER_PROC(flags)) {
+      if (count_bits(cpu_bitmap, sizeof(cpu_bitmap)) > 1)
+	 *dis_hdr = 1;
    }
 }
 
@@ -253,12 +299,12 @@ void set_loc_time(short curr)
  * Set timestamp string
  ***************************************************************************
 */
-void set_timestamp(short curr, char *cur_time, int len)
+void set_timestamp(int curr, char *cur_time, int len)
 {
-   set_loc_time(curr);
+   set_rectime(curr);
 
    /* Set cur_time date value */
-   strftime(cur_time, len, "%X", &loc_time);
+   strftime(cur_time, len, "%X", &rectime);
 }
 
 
@@ -267,7 +313,7 @@ void set_timestamp(short curr, char *cur_time, int len)
  * Core function used to display statistics
  ***************************************************************************
  */
-void write_stats_core(short prev, short curr, short dis, char *prev_string,
+void write_stats_core(int prev, int curr, int dis, char *prev_string,
 		      char *curr_string, unsigned int act,
 		      unsigned long long itv, unsigned long long g_itv,
 		      int disp_avg, int want_since_boot)
@@ -277,14 +323,8 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
       *fsi = &file_stats[curr],
       *fsj = &file_stats[prev];
 
-   /*
-    * Under very special circumstances, STDOUT may become unavailable,
-    * This is what we try to guess here
-    */
-   if (write(STDOUT_FILENO, "", 0) == -1) {
-      perror("stdout");
-      exit(6);
-   }
+   /* Test stdout */
+   TEST_STDOUT(STDOUT_FILENO);
 
    /* Print number of processes created per second */
    if (GET_PROC(act)) {
@@ -307,25 +347,24 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
    /* Print CPU usage */
    if (GET_CPU(act)) {
       if (dis)
-	 printf("\n%-11s       CPU     %%user     %%nice   %%system   "
+	 printf("\n%-11s     CPU     %%user     %%nice   %%system   "
 		"%%iowait    %%steal     %%idle\n",
 		prev_string);
 
       if (!WANT_PER_PROC(flags) ||
 	  (WANT_PER_PROC(flags) && WANT_ALL_PROC(flags))) {
 
-	 printf("%-11s       all", curr_string);
+	 printf("%-11s     all", curr_string);
 
-	 printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f",
+	 printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f\n",
 		ll_sp_value(fsj->cpu_user, fsi->cpu_user, g_itv),
 		ll_sp_value(fsj->cpu_nice, fsi->cpu_nice, g_itv),
 		ll_sp_value(fsj->cpu_system, fsi->cpu_system, g_itv),
 		ll_sp_value(fsj->cpu_iowait, fsi->cpu_iowait, g_itv),
-		ll_sp_value(fsj->cpu_steal, fsi->cpu_steal, g_itv));
-
-	 printf("    %6.2f\n",
-		fsi->cpu_idle < fsj->cpu_idle ? /* Handle buggy kernels */
-		0.0 : ll_sp_value(fsj->cpu_idle, fsi->cpu_idle, g_itv));
+		ll_sp_value(fsj->cpu_steal, fsi->cpu_steal, g_itv),
+		fsi->cpu_idle < fsj->cpu_idle ?
+		0.0 :
+		ll_sp_value(fsj->cpu_idle, fsi->cpu_idle, g_itv));
       }
 
       if (WANT_PER_PROC(flags) && file_hdr.sa_proc) {
@@ -337,21 +376,25 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	 for (i = 0; i < file_hdr.sa_proc; i++, sci++, scj++) {
 	    if (cpu_bitmap[i >> 3] & (1 << (i & 0x07))) {
 
-	       printf("%-11s       %3d", curr_string, i);
+	       printf("%-11s     %3d", curr_string, i);
 	
 	       /* Recalculate itv for current proc */
 	       pc_itv = get_per_cpu_interval(sci, scj);
-	
-	       printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f",
-		      ll_sp_value(scj->per_cpu_user, sci->per_cpu_user, pc_itv),
-		      ll_sp_value(scj->per_cpu_nice, sci->per_cpu_nice, pc_itv),
-		      ll_sp_value(scj->per_cpu_system, sci->per_cpu_system, pc_itv),
-		      ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv),
-		      ll_sp_value(scj->per_cpu_steal, sci->per_cpu_steal, pc_itv));
 
-	       printf("    %6.2f\n",
-		      sci->per_cpu_idle < scj->per_cpu_idle ?
-		      0.0 : ll_sp_value(scj->per_cpu_idle, sci->per_cpu_idle, pc_itv));
+	       if (!pc_itv)
+		  /* Current CPU is offline */
+		  printf("      0.00      0.00      0.00      0.00      0.00      0.00\n");
+	       else {
+		  printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f\n",
+			 ll_sp_value(scj->per_cpu_user, sci->per_cpu_user, pc_itv),
+			 ll_sp_value(scj->per_cpu_nice, sci->per_cpu_nice, pc_itv),
+			 ll_sp_value(scj->per_cpu_system, sci->per_cpu_system, pc_itv),
+			 ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv),
+			 ll_sp_value(scj->per_cpu_steal, sci->per_cpu_steal, pc_itv),
+			 sci->per_cpu_idle < scj->per_cpu_idle ?
+			 0.0 :
+			 ll_sp_value(scj->per_cpu_idle, sci->per_cpu_idle, pc_itv));
+	       }
 	    }
 	 }
       }
@@ -602,7 +645,7 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	 *sndj;
 
       if (dis)
-	 printf("\n%-11s     IFACE   rxpck/s   txpck/s   rxbyt/s   txbyt/s   "
+	 printf("\n%-11s     IFACE   rxpck/s   txpck/s    rxkB/s    txkB/s   "
 		"rxcmp/s   txcmp/s  rxmcst/s\n",
 		prev_string);
 
@@ -617,8 +660,8 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	 printf(" %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f\n",
 		S_VALUE(sndj->rx_packets, sndi->rx_packets, itv),
 		S_VALUE(sndj->tx_packets, sndi->tx_packets, itv),
-		S_VALUE(sndj->rx_bytes, sndi->rx_bytes, itv),
-		S_VALUE(sndj->tx_bytes, sndi->tx_bytes, itv),
+		S_VALUE(sndj->rx_bytes, sndi->rx_bytes, itv) / 1024,
+		S_VALUE(sndj->tx_bytes, sndi->tx_bytes, itv) / 1024,
 		S_VALUE(sndj->rx_compressed, sndi->rx_compressed, itv),
 		S_VALUE(sndj->tx_compressed, sndi->tx_compressed, itv),
 		S_VALUE(sndj->multicast, sndi->multicast, itv));
@@ -664,6 +707,7 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
       struct disk_stats
 	 *sdi = st_disk[curr],
 	 *sdj;
+      char *dev_name = NULL;
 
       if (dis)
 	 printf("\n%-11s       DEV       tps  rd_sec/s  wr_sec/s  avgrq-sz  "
@@ -688,10 +732,16 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	    ((sdi->rd_sect - sdj->rd_sect) + (sdi->wr_sect - sdj->wr_sect)) /
 	    ((double) (sdi->nr_ios - sdj->nr_ios)) : 0.0;
 
+	 if ((USE_PRETTY_OPTION(flags)) && (sdi->major == DEVMAP_MAJOR))
+	    dev_name = transform_devmapname(sdi->major, sdi->minor);
+	 
+	 if (!dev_name)
+	    dev_name = get_devname(sdi->major, sdi->minor, USE_PRETTY_OPTION(flags));
+	 
 	 printf("%-11s %9s %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f\n",
 		curr_string,
 		/* Confusion possible here between index and minor numbers */
-		get_devname(sdi->major, sdi->minor, USE_PRETTY_OPTION(flags)),
+		dev_name,
 		S_VALUE(sdj->nr_ios, sdi->nr_ios,  itv),
 		ll_s_value(sdj->rd_sect, sdi->rd_sect, itv),
 		ll_s_value(sdj->wr_sect, sdi->wr_sect, itv),
@@ -748,7 +798,7 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
  * Print statistics average
  ***************************************************************************
  */
-void write_stats_avg(int curr, short dis, unsigned int act, int read_from_file)
+void write_stats_avg(int curr, int dis, unsigned int act, int read_from_file)
 {
    unsigned long long itv, g_itv;
    char string[16];
@@ -756,21 +806,13 @@ void write_stats_avg(int curr, short dis, unsigned int act, int read_from_file)
      *fsi = &file_stats[curr];
 
    /* Interval value in jiffies */
-   g_itv = (file_stats[curr].uptime - file_stats[2].uptime) & 0xffffffff;
+   g_itv = get_interval(file_stats[2].uptime, file_stats[curr].uptime);
+
    if (file_hdr.sa_proc)
-      itv = (file_stats[curr].uptime0 - file_stats[2].uptime0) & 0xffffffff;
+      itv = get_interval(file_stats[2].uptime0, file_stats[curr].uptime0);
    else
       itv = g_itv;
 
-   if (!itv || !g_itv) { /* Should no longer happen with recent versions */
-      /*
-       * Aiee: null interval... This should only happen when reading stats
-       * from a system activity file with a far too big interval value...
-       */
-      fprintf(stderr, _("Please give a smaller interval value\n"));
-      exit(1);
-   }
-
    strcpy(string, _("Average:"));
    write_stats_core(2, curr, dis, string, string, act, itv, g_itv, TRUE, FALSE);
 
@@ -859,7 +901,7 @@ void write_stats_avg(int curr, short dis, unsigned int act, int read_from_file)
  * Print system statistics
  ***************************************************************************
  */
-int write_stats(short curr, short dis, unsigned int act, int read_from_file,
+int write_stats(int curr, int dis, unsigned int act, int read_from_file,
 		long *cnt, int use_tm_start, int use_tm_end, int reset,
 		int want_since_boot)
 {
@@ -870,8 +912,8 @@ int write_stats(short curr, short dis, unsigned int act, int read_from_file,
 
    /* Check time (1) */
    if (read_from_file) {
-      if (!next_slice(file_stats[2].uptime, file_stats[curr].uptime,
-		      &file_hdr, reset, interval))
+      if (!next_slice(file_stats[2].uptime0, file_stats[curr].uptime0,
+		      reset, interval))
 	 /* Not close enough to desired interval */
 	 return 0;
    }
@@ -882,7 +924,7 @@ int write_stats(short curr, short dis, unsigned int act, int read_from_file,
    set_timestamp(curr, cur_time[curr], 16);
 
    /* Check time (2) */
-   if (use_tm_start && (datecmp(&loc_time, &tm_start) < 0))
+   if (use_tm_start && (datecmp(&rectime, &tm_start) < 0))
      /* it's too soon... */
      return 0;
 
@@ -891,7 +933,7 @@ int write_stats(short curr, short dis, unsigned int act, int read_from_file,
 		 file_hdr.sa_proc, &itv, &g_itv);
 
    /* Check time (3) */
-   if (use_tm_end && (datecmp(&loc_time, &tm_end) > 0)) {
+   if (use_tm_end && (datecmp(&rectime, &tm_end) > 0)) {
       /* It's too late... */
       *cnt = 0;
       return 0;
@@ -1020,7 +1062,7 @@ int write_stats(short curr, short dis, unsigned int act, int read_from_file,
  * Display stats since system startup
  ***************************************************************************
  */
-void write_stats_startup(short curr)
+void write_stats_startup(int curr)
 {
    /* Set to 0 previous structures corresponding to boot time */
    memset(&file_stats[!curr], 0, FILE_STATS_SIZE);
@@ -1082,15 +1124,15 @@ int sa_read(void *buffer, int size)
  * Print a Linux restart message (contents of a DUMMY record)
  ***************************************************************************
  */
-void write_dummy(short curr, int use_tm_start, int use_tm_end)
+void write_dummy(int curr, int use_tm_start, int use_tm_end)
 {
    char cur_time[26];
 
    set_timestamp(curr, cur_time, 26);
 
    /* The RESTART message must be in the interval specified by -s/-e options */
-   if ((use_tm_start && (datecmp(&loc_time, &tm_start) < 0)) ||
-       (use_tm_end && (datecmp(&loc_time, &tm_end) > 0)))
+   if ((use_tm_start && (datecmp(&rectime, &tm_start) < 0)) ||
+       (use_tm_end && (datecmp(&rectime, &tm_end) > 0)))
       return;
 
    printf("\n%-11s       LINUX RESTART\n", cur_time);
@@ -1134,7 +1176,7 @@ void copy_structures(int dest, int src, int stype)
  * Read varying part of the statistics from a daily data file
  ***************************************************************************
  */
-void read_extra_stats(short curr, int ifd)
+void read_extra_stats(int curr, int ifd)
 {
    if (file_hdr.sa_proc)
       sa_fread(ifd, st_cpu[curr],
@@ -1160,10 +1202,10 @@ void read_extra_stats(short curr, int ifd)
 
 /*
  ***************************************************************************
- * Read a bunch of statistics sent by the data collector
+ * Read a bunch of statistics sent by the data collector (sadc)
  ***************************************************************************
  */
-void read_stat_bunch(short curr)
+void read_stat_bunch(int curr)
 {
    if (sa_read(&file_stats[curr], file_hdr.sa_st_size))
       exit(0);
@@ -1196,13 +1238,13 @@ void read_stat_bunch(short curr)
  * Read stats for current activity from file and display them.
  ***************************************************************************
  */
-void handle_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eosaf,
-			   int rows, unsigned int act, int *reset)
+void handle_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf,
+			   int rows, unsigned int act, int *reset, int nr_cpu, int nr_irq)
 {
-   short dis = 1;
+   int dis = 1;
    unsigned long lines;
    unsigned char rtype;
-   int davg, next;
+   int davg, next, inc = 1;
 
    if (lseek(ifd, fpos, SEEK_SET) < fpos) {
       perror("lseek");
@@ -1219,6 +1261,20 @@ void handle_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eos
    davg  = 0;
    *cnt  = count;
 
+   if (GET_NET_DEV(act) || GET_NET_EDEV(act))
+      inc = file_hdr.sa_iface;
+   else if (GET_DISK(act))
+      inc = file_hdr.sa_nr_disk;
+   else if (GET_CPU(act) && WANT_PER_PROC(flags)) {
+      inc = nr_cpu;
+      if (WANT_ALL_PROC(flags))
+	 inc++;	/* Nb of proc + "all" */
+   }
+   else if (GET_IRQ(act) && WANT_PER_PROC(flags))
+      inc = nr_cpu;
+   else if (GET_ONE_IRQ(act))
+      inc = nr_irq;
+
    do {
       /* Display count lines of stats */
       *eosaf = sa_fread(ifd, &file_stats[*curr],
@@ -1229,7 +1285,14 @@ void handle_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eos
 	 /* Read the extra fields since it's not a DUMMY record */
 	 read_extra_stats(*curr, ifd);
 
-      dis = !(lines++ % rows);
+      if ((lines >= rows) || !lines) {
+	 lines = 0;
+	 dis = 1;
+      }
+      else
+	 dis = 0;
+
+      lines += inc;
 
       if (!(*eosaf) && (rtype != R_DUMMY)) {
 
@@ -1243,7 +1306,7 @@ void handle_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eos
 	    *curr ^=1;
 	 }
 	 else
-	    lines--;
+	    lines -= inc;;
 	 *reset = 0;
       }
    }
@@ -1282,30 +1345,30 @@ void read_header_data(void)
  */
 void read_stats_from_file(char from_file[])
 {
-   short curr = 1;
+   int curr = 1;
    unsigned int act;
-   int ifd;
-   int rows = 23, eosaf = TRUE, reset = FALSE;
+   int ifd, nr_cpu, nr_irq;
+   int rows, eosaf = TRUE, reset = FALSE;
    long cnt = 1;
    off_t fpos;
 
-   if (!dis_hdr)
-      /* Get window size */
-      rows = get_win_height();
+   /* Get window size */
+   rows = get_win_height();
 
    /* Prepare file for reading */
    prep_file_for_reading(&ifd, from_file, &file_hdr, &sar_actflag, flags);
 
-   if ((GET_SERIAL(sar_actflag) && (file_hdr.sa_serial > 1)) ||
-       ((GET_NET_DEV(sar_actflag) || GET_NET_EDEV(sar_actflag)) && (file_hdr.sa_iface  > 1)) ||
-       (GET_DISK(sar_actflag) && (file_hdr.sa_nr_disk > 1)))
-      dis_hdr = 9;
+   nr_irq = count_bits(irq_bitmap, sizeof(irq_bitmap));
+   if (WANT_ALL_PROC(flags))
+      nr_cpu = file_hdr.sa_proc;
+   else
+      nr_cpu = count_bits(cpu_bitmap, sizeof(cpu_bitmap));
 
    /* Perform required allocations */
    allocate_structures(USE_SA_FILE);
 
    /* Print report header */
-   print_report_hdr(S_O_NONE, flags, &loc_time, &file_hdr);
+   print_report_hdr(flags, &rectime, &file_hdr);
 
    /* Read system statistics from file */
    do {
@@ -1326,12 +1389,12 @@ void read_stats_from_file(char from_file[])
 	     * So read now the extra fields.
 	     */
 	    read_extra_stats(0, ifd);
-	    set_loc_time(0);
+	    set_rectime(0);
 	 }
       }
       while ((file_stats[0].record_type == R_DUMMY) ||
-	     (tm_start.use && (datecmp(&loc_time, &tm_start) < 0)) ||
-	     (tm_end.use && (datecmp(&loc_time, &tm_end) >=0)));
+	     (tm_start.use && (datecmp(&rectime, &tm_start) < 0)) ||
+	     (tm_end.use && (datecmp(&rectime, &tm_end) >=0)));
 
       /* Save the first stats collected. Will be used to compute the average */
       copy_structures(2, 0, USE_SA_FILE);
@@ -1353,14 +1416,17 @@ void read_stats_from_file(char from_file[])
 	    if ((act == A_IRQ) && WANT_PER_PROC(flags) && WANT_ALL_PROC(flags)) {
 	       /* Distinguish -I SUM activity from IRQs per processor activity */
 	       flags &= ~S_F_PER_PROC;
-	       handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows, act, &reset);
+	       handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows, act, &reset,
+				     nr_cpu, nr_irq);
 	       flags |= S_F_PER_PROC;
 	       flags &= ~S_F_ALL_PROC;
-	       handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows, act, &reset);
+	       handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows, act, &reset,
+				     nr_cpu, nr_irq);
 	       flags |= S_F_ALL_PROC;
 	    }
 	    else
-	       handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows, act, &reset);
+	       handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows, act, &reset,
+				     nr_cpu, nr_irq);
 	 }
       }
 
@@ -1392,9 +1458,10 @@ void read_stats_from_file(char from_file[])
  */
 void read_stats(void)
 {
-   short curr = 1, dis = 1;
+   int curr = 1, dis = 1;
    unsigned long lines = 0;
-   unsigned int rows = 23, more = 1;
+   unsigned int rows = 23;
+   int dis_hdr = 0;
 
    /* Read stats header */
    read_header_data();
@@ -1409,35 +1476,24 @@ void read_stats(void)
     * Check that data corresponding to requested activities
     * are sent by the data collector.
     */
-   if (GET_SERIAL(sar_actflag) && !file_hdr.sa_serial) {
+   if (GET_SERIAL(sar_actflag) && !file_hdr.sa_serial)
       sar_actflag &= ~A_SERIAL;
-      dis_hdr--;
-   }
-   if ((sar_actflag & (A_NET_DEV + A_NET_EDEV)) && !file_hdr.sa_iface) {
-      if (GET_NET_DEV(sar_actflag) && GET_NET_EDEV(sar_actflag))
-	 dis_hdr--;
+
+   if ((GET_NET_DEV(sar_actflag) || GET_NET_EDEV(sar_actflag)) && !file_hdr.sa_iface)
       sar_actflag &= ~(A_NET_DEV + A_NET_EDEV);
-      dis_hdr--;
-   }
+
    if (GET_DISK(sar_actflag) && !file_hdr.sa_nr_disk) {
       sar_actflag &= ~A_DISK;
-      dis_hdr--;
    }
    if (!sar_actflag) {
       fprintf(stderr, _("Requested activities not available\n"));
       exit(1);
    }
 
-   if ((GET_SERIAL(sar_actflag) && (file_hdr.sa_serial > 1)) ||
-       ((GET_NET_DEV(sar_actflag) || GET_NET_EDEV(sar_actflag)) && (file_hdr.sa_iface  > 1)) ||
-       (GET_DISK(sar_actflag) && (file_hdr.sa_nr_disk > 1)))
-      dis_hdr = 9;
+   /* Determine if a stat line header has to be displayed */
+   check_line_hdr(&dis_hdr);
 
-   if (!dis_hdr) {
-      /* Get window size */
-      rows = get_win_height();
-      lines = rows;
-   }
+   lines = rows = get_win_height();
 
    /* Check use of -P option */
    prep_smp_option(file_hdr.sa_proc);
@@ -1451,18 +1507,11 @@ void read_stats(void)
    allocate_structures(USE_SADC);
 
    /* Print report header */
-   print_report_hdr(S_O_NONE, flags, &loc_time, &file_hdr);
+   print_report_hdr(flags, &rectime, &file_hdr);
 
    /* Read system statistics sent by the data collector */
    read_stat_bunch(0);
 
-   if (!dis_hdr) {
-      if (file_hdr.sa_proc)
-	 more = 2 + file_hdr.sa_proc;
-      if (pid_nr)
-	 more = pid_nr;
-   }
-
    if (!interval)
       /* Display stats since boot time and exit */
       write_stats_startup(0);
@@ -1481,7 +1530,7 @@ void read_stats(void)
 	 dis = lines / rows;
 	 if (dis)
 	    lines %= rows;
-	 lines += more;
+	 lines++;
       }
       write_stats(curr, dis, sar_actflag, USE_SADC, &count, NO_TM_START,
 		  tm_end.use, NO_RESET, ST_IMMEDIATE);
@@ -1518,8 +1567,11 @@ int main(int argc, char **argv)
    char from_file[MAX_FILE_LEN], to_file[MAX_FILE_LEN];
    char ltemp[20];
 
+   /* Get HZ */
+   get_HZ();
+
    /* Compute page shift in kB */
-   kb_shift = get_kb_shift();
+   get_kb_shift();
 
    from_file[0] = to_file[0] = '\0';
 
@@ -1538,10 +1590,8 @@ int main(int argc, char **argv)
 
       if (!strcmp(argv[opt], "-I")) {
 	 if (argv[++opt]) {
-	    dis_hdr++;
 	    /* Parse -I option */
-	    if (parse_sar_I_opt(argv, &opt, &sar_actflag, &dis_hdr,
-				irq_bitmap))
+	    if (parse_sar_I_opt(argv, &opt, &sar_actflag, irq_bitmap))
 	       usage(argv[0]);
 	 }
 	 else
@@ -1550,7 +1600,7 @@ int main(int argc, char **argv)
 
       else if (!strcmp(argv[opt], "-P")) {
 	 /* Parse -P option */
-	 if (parse_sa_P_opt(argv, &opt, &flags, &dis_hdr, cpu_bitmap))
+	 if (parse_sa_P_opt(argv, &opt, &flags, cpu_bitmap))
 	    usage(argv[0]);
       }
 
@@ -1572,12 +1622,8 @@ int main(int argc, char **argv)
 	    strncpy(from_file, argv[opt++], MAX_FILE_LEN);
 	    from_file[MAX_FILE_LEN - 1] = '\0';
 	 }
-	 else {
-	    get_localtime(&loc_time);
-	    snprintf(from_file, MAX_FILE_LEN,
-		     "%s/sa%02d", SA_DIR, loc_time.tm_mday);
-	    from_file[MAX_FILE_LEN - 1] = '\0';
-	 }
+	 else
+	    set_default_file(&rectime, from_file);
       }
 
       else if (!strcmp(argv[opt], "-s")) {
@@ -1603,14 +1649,12 @@ int main(int argc, char **argv)
 
       else if (!strcmp(argv[opt], "-x") || !strcmp(argv[opt], "-X")) {
 	 if (argv[++opt]) {
-	    dis_hdr++;
 	    if (!strcmp(argv[opt], K_SADC)) {
 	       strcpy(ltemp, K_SELF);
 	       opt++;
 	    }
 	    else if (!strcmp(argv[opt], K_ALL)) {
 	       if (args_idx < MAX_ARGV_NR - 7) {
-		  dis_hdr = 9;
 		  salloc(args_idx++, argv[opt - 1]);	/* "-x" or "-X" */
 		  salloc(args_idx++, K_ALL);
 		  if (!strcmp(argv[opt - 1], "-x"))
@@ -1651,9 +1695,8 @@ int main(int argc, char **argv)
 
       else if (!strcmp(argv[opt], "-n")) {
 	 if (argv[++opt]) {
-	    dis_hdr++;
 	    /* Parse option -n */
-	    if (parse_sar_n_opt(argv, &opt, &sar_actflag, &dis_hdr))
+	    if (parse_sar_n_opt(argv, &opt, &sar_actflag))
 	       usage(argv[0]);
 	 }
 	 else
@@ -1662,7 +1705,7 @@ int main(int argc, char **argv)
 
       else if (!strncmp(argv[opt], "-", 1)) {
 	 /* Other options not previously tested */
-	 if (parse_sar_opt(argv, opt, &sar_actflag, &flags, &dis_hdr, C_SAR,
+	 if (parse_sar_opt(argv, opt, &sar_actflag, &flags, C_SAR,
 			   irq_bitmap, cpu_bitmap))
 	    usage(argv[0]);
 	 opt++;
@@ -1693,12 +1736,8 @@ int main(int argc, char **argv)
 
    /* 'sar' is equivalent to 'sar -f' */
    if ((argc == 1) ||
-       ((interval < 0) && !from_file[0] && !to_file[0])) {
-      get_localtime(&loc_time);
-      snprintf(from_file, MAX_FILE_LEN,
-	       "%s/sa%02d", SA_DIR, loc_time.tm_mday);
-      from_file[MAX_FILE_LEN - 1] = '\0';
-   }
+       ((interval < 0) && !from_file[0] && !to_file[0]))
+      set_default_file(&rectime, from_file);
 
    /*
     * Check option dependencies
@@ -1735,14 +1774,12 @@ int main(int argc, char **argv)
    }
 
    /* Default is CPU activity... */
-   if (!sar_actflag) {
+   if (!sar_actflag)
       /*
        * Still OK even when reading stats from a file
        * since A_CPU activity is always recorded.
        */
       sar_actflag |= A_CPU;
-      dis_hdr++;
-   }
 
    /* ---Reading stats from file */
    if (from_file[0]) {
